@@ -1,11 +1,11 @@
 /* Formatted output to strings in C locale.
-   Copyright (C) 2004, 2006-2020 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2006-2022 Free Software Foundation, Inc.
    Written by Simon Josefsson and Paul Eggert.
    Modified for C locale by Ben Pfaff.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
+   the Free Software Foundation, either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -19,7 +19,7 @@
 #include <config.h>
 
 /* Specification.  */
-#include <stdio.h>
+#include "c-snprintf.h"
 
 #include <errno.h>
 #include <limits.h>
