@@ -112,7 +112,7 @@ struct casereader_class
 
 struct casereader *
 casereader_create_sequential (const struct taint *,
-                              const struct caseproto *, casenumber case_cnt,
+                              const struct caseproto *, casenumber n_cases,
                               const struct casereader_class *, void *);
 
 void *casereader_dynamic_cast (struct casereader *, const struct casereader_class *);
@@ -150,17 +150,17 @@ struct casereader_random_class
 
     /* Mandatory.
 
-       A call to this function tells the callee that the CNT
+       A call to this function tells the callee that the N
        cases at the beginning of READER will never be read again.
        The casereader implementation should free any resources
        associated with those cases.  After this function returns,
        the IDX argument in future calls to the "read" function
        will be relative to remaining cases. */
-    void (*advance) (struct casereader *reader, void *aux, casenumber cnt);
+    void (*advance) (struct casereader *reader, void *aux, casenumber n);
   };
 
 struct casereader *
-casereader_create_random (const struct caseproto *, casenumber case_cnt,
+casereader_create_random (const struct caseproto *, casenumber n_cases,
                           const struct casereader_random_class *, void *aux);
 
 #endif /* data/casereader-provider.h */
