@@ -27,9 +27,8 @@ struct file_handle;
 
 int encrypted_file_open (struct encrypted_file **, const struct file_handle *);
 bool encrypted_file_unlock (struct encrypted_file *, const char *password);
+bool encrypted_file_unlock__ (struct encrypted_file *, const char *password);
 size_t encrypted_file_read (struct encrypted_file *, void *, size_t);
 int encrypted_file_close (struct encrypted_file *);
 
-bool encrypted_file_is_sav (const struct encrypted_file *);
-
 #endif /* encrypted-file.h */
