@@ -19,20 +19,7 @@
 
 #include <stdbool.h>
 
-struct casereader;
-struct dictionary;
-struct spreadsheet_read_info;
-struct spreadsheet_read_options;
-
 struct spreadsheet *gnumeric_probe (const char *filename, bool report_errors);
 
-const char * gnumeric_get_sheet_name (struct spreadsheet *s, int n);
-char * gnumeric_get_sheet_range (struct spreadsheet *s, int n);
-
-struct casereader * gnumeric_make_reader (struct spreadsheet *spreadsheet,
-					  const struct spreadsheet_read_options *opts);
-
-void gnumeric_unref (struct spreadsheet *r);
-
 
 #endif
