@@ -233,7 +233,7 @@ psql_open_reader (struct psql_read_info *info, struct dictionary **dict)
   casenumber n_cases = CASENUMBER_MAX;
   const char *encoding;
 
-  struct psql_reader *r = xzalloc (sizeof *r);
+  struct psql_reader *r = XZALLOC (struct psql_reader);
   struct string query ;
 
   r->conn = PQconnectdb (info->conninfo);
@@ -286,7 +286,8 @@ psql_open_reader (struct psql_read_info *info, struct dictionary **dict)
 	}
     }
 
-  r->postgres_epoch = calendar_gregorian_to_offset (2000, 1, 1, NULL);
+  r->postgres_epoch = calendar_gregorian_to_offset (
+    2000, 1, 1, settings_get_fmt_settings (), NULL);
 
   {
     const int enc = PQclientEncoding (r->conn);
@@ -371,7 +372,7 @@ psql_open_reader (struct psql_read_info *info, struct dictionary **dict)
   for (i = 0 ; i < n_fields ; ++i)
     {
       struct variable *var;
-      struct fmt_spec fmt = {FMT_F, 8, 2};
+      struct fmt_spec fmt = { .type = FMT_F, .w = 8, .d = 2 };
       Oid type = PQftype (qres, i);
       int width = 0;
       int length ;
@@ -614,7 +615,7 @@ set_value (struct psql_reader *r)
 	case INTERVALOID:
 	case TIMESTAMPTZOID:
 	case TIMETZOID:
-	  if (i < r->vmapsize && var_get_dict_index(v) + 1 < dict_get_var_cnt (r->dict))
+	  if (i < r->vmapsize && var_get_dict_index(v) + 1 < dict_get_n_vars (r->dict))
 	    {
 	      const struct variable *v1 = NULL;
 	      v1 = dict_get_var (r->dict, var_get_dict_index (v) + 1);
