@@ -16,7 +16,6 @@
 
 #include <config.h>
 
-#include <stddef.h>
 #include <stdlib.h>
 
 #include "data/dataset.h"
@@ -25,7 +24,6 @@
 #include "data/value-labels.h"
 #include "data/variable.h"
 #include "language/command.h"
-#include "language/control/control-stack.h"
 #include "language/lexer/lexer.h"
 #include "libpspp/message.h"
 #include "libpspp/str.h"
