@@ -40,9 +40,10 @@
 #include "libpspp/misc.h"
 #include "libpspp/pool.h"
 #include "libpspp/u8-line.h"
+#include "output/driver.h"
 #include "output/pivot-table.h"
 #include "output/table.h"
-#include "output/text-item.h"
+#include "output/output-item.h"
 
 #include "gl/xalloc.h"
 
@@ -86,7 +87,7 @@ struct print_trns
     const char *encoding;       /* Encoding to use for output. */
     struct dfm_writer *writer;	/* Output file, NULL=listing file. */
     struct ll_list specs;       /* List of struct prt_out_specs. */
-    size_t record_cnt;          /* Number of records to write. */
+    size_t n_records;           /* Number of records to write. */
   };
 
 enum which_formats
@@ -95,13 +96,16 @@ enum which_formats
     WRITE
   };
 
+static const struct trns_class print_binary_trns_class;
+static const struct trns_class print_text_trns_class;
+
 static int internal_cmd_print (struct lexer *, struct dataset *ds,
 			       enum which_formats, bool eject);
-static trns_proc_func print_text_trns_proc, print_binary_trns_proc;
-static trns_free_func print_trns_free;
 static bool parse_specs (struct lexer *, struct pool *tmp_pool, struct print_trns *,
 			 struct dictionary *dict, enum which_formats);
 static void dump_table (struct print_trns *);
+
+static bool print_trns_free (void *trns_);
 
 /* Basic parsing. */
 
@@ -143,7 +147,7 @@ internal_cmd_print (struct lexer *lexer, struct dataset *ds,
   trns = pool_create_container (struct print_trns, pool);
   trns->eject = eject;
   trns->writer = NULL;
-  trns->record_cnt = 0;
+  trns->n_records = 0;
   ll_init (&trns->specs);
 
   tmp_pool = pool_create_subpool (trns->pool);
@@ -174,9 +178,9 @@ internal_cmd_print (struct lexer *lexer, struct dataset *ds,
 	{
 	  lex_match (lexer, T_EQUALS);
 	  lex_match (lexer, T_LPAREN);
-	  if (!lex_force_int (lexer))
+	  if (!lex_force_int_range (lexer, "RECORDS", 0, INT_MAX))
 	    goto error;
-	  trns->record_cnt = lex_integer (lexer);
+	  trns->n_records = lex_integer (lexer);
 	  lex_get (lexer);
 	  lex_match (lexer, T_RPAREN);
 	}
@@ -238,11 +242,9 @@ internal_cmd_print (struct lexer *lexer, struct dataset *ds,
     dump_table (trns);
 
   /* Put the transformation in the queue. */
-  add_transformation (ds,
-                      (binary
-                       ? print_binary_trns_proc
-                       : print_text_trns_proc),
-                      print_trns_free, trns);
+  add_transformation (ds, (binary
+                           ? &print_binary_trns_class
+                           : &print_text_trns_class), trns);
 
   pool_destroy (tmp_pool);
   fh_unref (fh);
@@ -276,7 +278,7 @@ parse_specs (struct lexer *lexer, struct pool *tmp_pool, struct print_trns *trns
 
   if (lex_token (lexer) == T_ENDCMD)
     {
-      trns->record_cnt = 1;
+      trns->n_records = 1;
       return true;
     }
 
@@ -298,11 +300,11 @@ parse_specs (struct lexer *lexer, struct pool *tmp_pool, struct print_trns *trns
       lex_match (lexer, T_COMMA);
     }
 
-  if (trns->record_cnt != 0 && trns->record_cnt != record)
+  if (trns->n_records != 0 && trns->n_records != record)
     msg (SW, _("Output calls for %d records but %zu specified on RECORDS "
                "subcommand."),
-         record, trns->record_cnt);
-  trns->record_cnt = record;
+         record, trns->n_records);
+  trns->n_records = record;
 
   return true;
 }
@@ -353,19 +355,19 @@ parse_variable_argument (struct lexer *lexer, const struct dictionary *dict,
                          enum which_formats which_formats)
 {
   const struct variable **vars;
-  size_t var_cnt, var_idx;
+  size_t n_vars, var_idx;
   struct fmt_spec *formats, *f;
-  size_t format_cnt;
+  size_t n_formats;
   bool add_space;
 
   if (!parse_variables_const_pool (lexer, tmp_pool, dict,
-			     &vars, &var_cnt, PV_DUPLICATE))
+			     &vars, &n_vars, PV_DUPLICATE))
     return false;
 
   if (lex_is_number (lexer) || lex_token (lexer) == T_LPAREN)
     {
-      if (!parse_var_placements (lexer, tmp_pool, var_cnt, FMT_FOR_OUTPUT,
-                                 &formats, &format_cnt))
+      if (!parse_var_placements (lexer, tmp_pool, n_vars, FMT_FOR_OUTPUT,
+                                 &formats, &n_formats))
         return false;
       add_space = false;
     }
@@ -375,9 +377,9 @@ parse_variable_argument (struct lexer *lexer, const struct dictionary *dict,
 
       lex_match (lexer, T_ASTERISK);
 
-      formats = pool_nmalloc (tmp_pool, var_cnt, sizeof *formats);
-      format_cnt = var_cnt;
-      for (i = 0; i < var_cnt; i++)
+      formats = pool_nmalloc (tmp_pool, n_vars, sizeof *formats);
+      n_formats = n_vars;
+      for (i = 0; i < n_vars; i++)
         {
           const struct variable *v = vars[i];
           formats[i] = (which_formats == PRINT
@@ -388,7 +390,7 @@ parse_variable_argument (struct lexer *lexer, const struct dictionary *dict,
     }
 
   var_idx = 0;
-  for (f = formats; f < &formats[format_cnt]; f++)
+  for (f = formats; f < &formats[n_formats]; f++)
     if (!execute_placement_format (f, record, column))
       {
         const struct variable *var;
@@ -419,7 +421,7 @@ parse_variable_argument (struct lexer *lexer, const struct dictionary *dict,
 
         *column += f->w + add_space;
       }
-  assert (var_idx == var_cnt);
+  assert (var_idx == n_vars);
 
   return true;
 }
@@ -450,7 +452,7 @@ dump_table (struct print_trns *trns)
                         pivot_value_new_integer (spec->record));
       int last_column = spec->first_column + spec->format.w - 1;
       pivot_table_put2 (table, 1, row, pivot_value_new_user_text_nocopy (
-                          xasprintf ("%3d-%3d",
+                          xasprintf ("%d-%d",
                                      spec->first_column, last_column)));
 
       char fmt_string[FMT_STRING_LEN_MAX + 1];
@@ -461,7 +463,7 @@ dump_table (struct print_trns *trns)
   int row = pivot_category_create_leaf (
     variables->root, pivot_value_new_text (N_("N of Records")));
   pivot_table_put2 (table, 0, row,
-                    pivot_value_new_integer (trns->record_cnt));
+                    pivot_value_new_integer (trns->n_records));
 
   pivot_table_submit (table);
 }
@@ -473,7 +475,7 @@ static void print_text_flush_records (struct print_trns *, struct u8_line *,
                                       bool *eject, int *record);
 
 /* Performs the transformation inside print_trns T on case C. */
-static int
+static enum trns_result
 print_text_trns_proc (void *trns_, struct ccase **c,
                       casenumber case_num UNUSED)
 {
@@ -504,7 +506,7 @@ print_text_trns_proc (void *trns_, struct ccase **c,
               char *s;
 
               s = data_out (input, var_get_encoding (spec->var),
-                            &spec->format);
+                            &spec->format, settings_get_fmt_settings ());
               len = strlen (s);
               width = u8_width (CHAR_CAST (const uint8_t *, s), len, UTF8);
               x1 = x0 + width;
@@ -530,7 +532,7 @@ print_text_trns_proc (void *trns_, struct ccase **c,
                        ds_data (s), ds_length (s));
         }
     }
-  print_text_flush_records (trns, &line, trns->record_cnt + 1,
+  print_text_flush_records (trns, &line, trns->n_records + 1,
                             &eject, &record);
   u8_line_destroy (&line);
 
@@ -555,14 +557,14 @@ print_text_flush_records (struct print_trns *trns, struct u8_line *line,
         {
           *eject = false;
           if (trns->writer == NULL)
-            text_item_submit (text_item_create (TEXT_ITEM_EJECT_PAGE, ""));
+            output_item_submit (page_break_item_create ());
           else
             leader = '1';
         }
       *u8_line_reserve (line, 0, 1, 1) = leader;
 
       if (trns->writer == NULL)
-        table_output_text (TAB_FIX, ds_cstr (&line->s) + 1);
+        output_log ("%s", ds_cstr (&line->s) + 1);
       else
         {
           size_t len = ds_length (&line->s);
@@ -574,14 +576,7 @@ print_text_flush_records (struct print_trns *trns, struct u8_line *line,
               len--;
             }
 
-          if (is_encoding_utf8 (trns->encoding))
-            dfm_put_record (trns->writer, s, len);
-          else
-            {
-              char *recoded = recode_string (trns->encoding, UTF8, s, len);
-              dfm_put_record (trns->writer, recoded, strlen (recoded));
-              free (recoded);
-            }
+          dfm_put_record_utf8 (trns->writer, s, len);
         }
     }
 }
@@ -593,7 +588,7 @@ static void print_binary_flush_records (struct print_trns *,
                                         bool *eject, int *record);
 
 /* Performs the transformation inside print_trns T on case C. */
-static int
+static enum trns_result
 print_binary_trns_proc (void *trns_, struct ccase **c,
                         casenumber case_num UNUSED)
 {
@@ -616,7 +611,8 @@ print_binary_trns_proc (void *trns_, struct ccase **c,
           const union value *input = case_data (*c, spec->var);
           if (!spec->sysmis_as_spaces || input->f != SYSMIS)
             data_out_recode (input, var_get_encoding (spec->var),
-                             &spec->format, &line, trns->encoding);
+                             &spec->format, settings_get_fmt_settings (),
+                             &line, trns->encoding);
           else
             ds_put_byte_multiple (&line, encoded_space, spec->format.w);
           if (spec->add_space)
@@ -635,7 +631,7 @@ print_binary_trns_proc (void *trns_, struct ccase **c,
             }
         }
     }
-  print_binary_flush_records (trns, &line, trns->record_cnt + 1,
+  print_binary_flush_records (trns, &line, trns->n_records + 1,
                               &eject, &record);
   ds_destroy (&line);
 
@@ -690,3 +686,15 @@ print_trns_free (void *trns_)
   return ok;
 }
 
+static const struct trns_class print_binary_trns_class = {
+  .name = "PRINT",
+  .execute = print_binary_trns_proc,
+  .destroy = print_trns_free,
+};
+
+static const struct trns_class print_text_trns_class = {
+  .name = "PRINT",
+  .execute = print_text_trns_proc,
+  .destroy = print_trns_free,
+};
+
