@@ -79,11 +79,10 @@ parse_group (struct lexer *lexer, struct dictionary *dict,
              enum mrset_type type)
 {
   const char *subcommand_name = type == MRSET_MD ? "MDGROUP" : "MCGROUP";
-  struct mrset *mrset;
   bool labelsource_varlabel;
   bool has_value;
 
-  mrset = xzalloc (sizeof *mrset);
+  struct mrset *mrset = XZALLOC (struct mrset);
   mrset->type = type;
   mrset->cat_source = MRSET_VARLABELS;
 
@@ -421,7 +420,8 @@ parse_group (struct lexer *lexer, struct dictionary *dict,
                       if (!c->warned && utf8_strcasecmp (c->label, label))
                         {
                           char *s = data_out (value, var_get_encoding (var),
-                                              var_get_print_format (var));
+                                              var_get_print_format (var),
+                                              settings_get_fmt_settings ());
                           c->warned = true;
                           msg (SW, _("Variables specified on MCGROUP should "
                                      "have the same categories, but %s and %s "
