@@ -19,10 +19,13 @@
 
 #include <stdbool.h>
 
+#include "data/format.h"
+
 struct lexer;
 
-bool parse_abstract_format_specifier (struct lexer *, char *type,
-                                      int *width, int *decimals);
+bool parse_abstract_format_specifier (struct lexer *,
+                                      char type[FMT_TYPE_LEN_MAX + 1],
+                                      uint16_t *width, uint8_t *decimals);
 
 enum fmt_type ;
 struct fmt_spec;
