@@ -80,13 +80,13 @@ cochran_execute (const struct dataset *ds,
   for (; (c = casereader_read (input)); case_unref (c))
     {
       double case_hits = 0.0;
-      const double w = weight ? case_data (c, weight)->f: 1.0;
+      const double w = weight ? case_num (c, weight) : 1.0;
       for (v = 0; v < ct->n_vars; ++v)
 	{
 	  const struct variable *var = ct->vars[v];
 	  const union value *val = case_data (c, var);
 
-	  if (var_is_value_missing (var, val, exclude))
+	  if (var_is_value_missing (var, val) & exclude)
 	    continue;
 
 	  if (ch.success == SYSMIS)
