@@ -153,10 +153,10 @@ sign_execute (const struct dataset *ds,
 	  const union value *value1 = case_data (c, (*vp)[1]);
 	  const double diff = value0->f - value1->f;
 
-	  if (var_is_value_missing ((*vp)[0], value0, exclude))
+	  if (var_is_value_missing ((*vp)[0], value0) & exclude)
 	    continue;
 
-	  if (var_is_value_missing ((*vp)[1], value1, exclude))
+	  if (var_is_value_missing ((*vp)[1], value1) & exclude)
 	    continue;
 
 	  if (diff > 0)
