@@ -61,7 +61,7 @@ cmd_output (struct lexer *lexer, struct dataset *ds UNUSED)
       else if (lex_match_id (lexer, "TABLECELLS"))
 	{
           string_set_clear (&rc_names);
-	  struct fmt_spec fmt = { 0, 0, 0 };
+	  struct fmt_spec fmt = { .type = 0 };
 
 	  while (lex_token (lexer) != T_SLASH &&
 		 lex_token (lexer) != T_ENDCMD)
@@ -86,8 +86,8 @@ cmd_output (struct lexer *lexer, struct dataset *ds UNUSED)
 	      else if (lex_match_id (lexer, "FORMAT"))
 		{
 		  char type[FMT_TYPE_LEN_MAX + 1];
-		  int width = -1;
-		  int decimals = -1;
+		  uint16_t width;
+		  uint8_t decimals;
 
 		  if (! lex_force_match (lexer, T_EQUALS))
 		    goto error;
