@@ -28,11 +28,8 @@
 #include <uniwidth.h>
 
 #ifdef HAVE_TERMIOS_H
-# include <termios.h>
-#endif
-
-#ifdef GWINSZ_IN_SYS_IOCTL
 # include <sys/ioctl.h>
+# include <termios.h>
 #endif
 
 #include "data/file-name.h"
@@ -47,14 +44,14 @@
 #include "libpspp/u8-line.h"
 #include "libpspp/version.h"
 #include "output/ascii.h"
-#include "output/cairo.h"
-#include "output/chart-item-provider.h"
+#include "output/cairo-chart.h"
+#include "output/chart-provider.h"
 #include "output/driver-provider.h"
-#include "output/message-item.h"
 #include "output/options.h"
+#include "output/pivot-output.h"
+#include "output/pivot-table.h"
 #include "output/render.h"
-#include "output/table-item.h"
-#include "output/text-item.h"
+#include "output/output-item.h"
 
 #include "gl/minmax.h"
 #include "gl/xalloc.h"
@@ -70,76 +67,179 @@
 enum
   {
     ASCII_LINE_NONE,
+    ASCII_LINE_DASHED,
     ASCII_LINE_SINGLE,
     ASCII_LINE_DOUBLE,
     ASCII_N_LINES
   };
 
-#define N_BOX (ASCII_N_LINES * ASCII_N_LINES \
-               * ASCII_N_LINES * ASCII_N_LINES)
-
-static const ucs4_t ascii_box_chars[N_BOX] =
+struct box_chars
   {
-    ' ', '|', '#',
-    '-', '+', '#',
-    '=', '#', '#',
-    '|', '|', '#',
-    '+', '+', '#',
-    '#', '#', '#',
-    '#', '#', '#',
-    '#', '#', '#',
-    '#', '#', '#',
-    '-', '+', '#',
-    '-', '+', '#',
-    '#', '#', '#',
-    '+', '+', '#',
-    '+', '+', '#',
-    '#', '#', '#',
-    '#', '#', '#',
-    '#', '#', '#',
-    '#', '#', '#',
-    '=', '#', '#',
-    '#', '#', '#',
-    '=', '#', '#',
-    '#', '#', '#',
-    '#', '#', '#',
-    '#', '#', '#',
-    '#', '#', '#',
-    '#', '#', '#',
-    '#', '#', '#',
+    ucs4_t c[ASCII_N_LINES][ASCII_N_LINES][ASCII_N_LINES][ASCII_N_LINES];
   };
 
-static const ucs4_t unicode_box_chars[N_BOX] =
-  {
-    0x0020, 0x2575, 0x2551,
-    0x2574, 0x256f, 0x255c,
-    0x2550, 0x255b, 0x255d,
-    0x2577, 0x2502, 0x2551,
-    0x256e, 0x2524, 0x2562,
-    0x2555, 0x2561, 0x2563,
-    0x2551, 0x2551, 0x2551,
-    0x2556, 0x2562, 0x2562,
-    0x2557, 0x2563, 0x2563,
-    0x2576, 0x2570, 0x2559,
-    0x2500, 0x2534, 0x2568,
-    0x2550, 0x2567, 0x2569,
-    0x256d, 0x251c, 0x255f,
-    0x252c, 0x253c, 0x256a,
-    0x2564, 0x256a, 0x256c,
-    0x2553, 0x255f, 0x255f,
-    0x2565, 0x256b, 0x256b,
-    0x2566, 0x256c, 0x256c,
-    0x2550, 0x2558, 0x255a,
-    0x2550, 0x2567, 0x2569,
-    0x2550, 0x2567, 0x2569,
-    0x2552, 0x255e, 0x2560,
-    0x2564, 0x256a, 0x256c,
-    0x2564, 0x256a, 0x256c,
-    0x2554, 0x2560, 0x2560,
-    0x2560, 0x256c, 0x256c,
-    0x2566, 0x256c, 0x256c,
+static const struct box_chars *
+get_ascii_box (void)
+{
+  enum {
+    _ = ASCII_LINE_NONE,
+    d = ASCII_LINE_DASHED,
+    S = ASCII_LINE_SINGLE,
+    D = ASCII_LINE_DOUBLE,
   };
 
+  static const struct box_chars ascii_box =
+    {
+      /* r  b  l   t:  _    d    S    D */
+      .c[_][_][_] = { ' ', '|', '|', '#', },
+      .c[_][_][d] = { '-', '+', '+', '#', },
+      .c[_][_][S] = { '-', '+', '+', '#', },
+      .c[_][_][D] = { '=', '#', '#', '#', },
+      .c[_][d][_] = { '|', '|', '|', '#', },
+      .c[_][d][d] = { '+', '+', '+', '#', },
+      .c[_][d][S] = { '+', '+', '+', '#', },
+      .c[_][d][D] = { '#', '#', '#', '#', },
+      .c[_][S][_] = { '|', '|', '|', '#', },
+      .c[_][S][d] = { '+', '+', '+', '#', },
+      .c[_][S][S] = { '+', '+', '+', '#', },
+      .c[_][S][D] = { '#', '#', '#', '#', },
+      .c[_][D][_] = { '#', '#', '#', '#', },
+      .c[_][D][d] = { '#', '#', '#', '#', },
+      .c[_][D][S] = { '#', '#', '#', '#', },
+      .c[_][D][D] = { '#', '#', '#', '#', },
+      .c[d][_][_] = { '-', '+', '+', '#', },
+      .c[d][_][d] = { '-', '+', '+', '#', },
+      .c[d][_][S] = { '-', '+', '+', '#', },
+      .c[d][_][D] = { '#', '#', '#', '#', },
+      .c[d][d][_] = { '+', '+', '+', '#', },
+      .c[d][d][d] = { '+', '+', '+', '#', },
+      .c[d][d][S] = { '+', '+', '+', '#', },
+      .c[d][d][D] = { '#', '#', '#', '#', },
+      .c[d][S][_] = { '+', '+', '+', '#', },
+      .c[d][S][d] = { '+', '+', '+', '#', },
+      .c[d][S][S] = { '+', '+', '+', '#', },
+      .c[d][S][D] = { '#', '#', '#', '#', },
+      .c[d][D][_] = { '#', '#', '#', '#', },
+      .c[d][D][d] = { '#', '#', '#', '#', },
+      .c[d][D][S] = { '#', '#', '#', '#', },
+      .c[d][D][D] = { '#', '#', '#', '#', },
+      .c[S][_][_] = { '-', '+', '+', '#', },
+      .c[S][_][d] = { '-', '+', '+', '#', },
+      .c[S][_][S] = { '-', '+', '+', '#', },
+      .c[S][_][D] = { '#', '#', '#', '#', },
+      .c[S][d][_] = { '+', '+', '+', '#', },
+      .c[S][d][d] = { '+', '+', '+', '#', },
+      .c[S][d][S] = { '+', '+', '+', '#', },
+      .c[S][d][D] = { '#', '#', '#', '#', },
+      .c[S][S][_] = { '+', '+', '+', '#', },
+      .c[S][S][d] = { '+', '+', '+', '#', },
+      .c[S][S][S] = { '+', '+', '+', '#', },
+      .c[S][S][D] = { '#', '#', '#', '#', },
+      .c[S][D][_] = { '#', '#', '#', '#', },
+      .c[S][D][d] = { '#', '#', '#', '#', },
+      .c[S][D][S] = { '#', '#', '#', '#', },
+      .c[S][D][D] = { '#', '#', '#', '#', },
+      .c[D][_][_] = { '=', '#', '#', '#', },
+      .c[D][_][d] = { '#', '#', '#', '#', },
+      .c[D][_][S] = { '#', '#', '#', '#', },
+      .c[D][_][D] = { '=', '#', '#', '#', },
+      .c[D][d][_] = { '#', '#', '#', '#', },
+      .c[D][d][d] = { '#', '#', '#', '#', },
+      .c[D][d][S] = { '#', '#', '#', '#', },
+      .c[D][d][D] = { '#', '#', '#', '#', },
+      .c[D][S][_] = { '#', '#', '#', '#', },
+      .c[D][S][d] = { '#', '#', '#', '#', },
+      .c[D][S][S] = { '#', '#', '#', '#', },
+      .c[D][S][D] = { '#', '#', '#', '#', },
+      .c[D][D][_] = { '#', '#', '#', '#', },
+      .c[D][D][d] = { '#', '#', '#', '#', },
+      .c[D][D][S] = { '#', '#', '#', '#', },
+      .c[D][D][D] = { '#', '#', '#', '#', },
+    };
+  return &ascii_box;
+}
+
+static const struct box_chars *
+get_unicode_box (void)
+{
+  enum {
+    _ = ASCII_LINE_NONE,
+    d = ASCII_LINE_DASHED,
+    S = ASCII_LINE_SINGLE,
+    D = ASCII_LINE_DOUBLE,
+  };
+
+  static const struct box_chars unicode_box =
+    {
+      /* r  b  l   t:   _       d       S       D */
+      .c[_][_][_] = { 0x0020, 0x2575, 0x2575, 0x2551, }, /*  ╵╵║ */
+      .c[_][_][d] = { 0x2574, 0x256f, 0x256f, 0x255c, }, /* ╴╯╯╜ */
+      .c[_][_][S] = { 0x2574, 0x256f, 0x256f, 0x255c, }, /* ╴╯╯╜ */
+      .c[_][_][D] = { 0x2550, 0x255b, 0x255b, 0x255d, }, /* ═╛╛╝ */
+      .c[_][S][_] = { 0x2577, 0x2502, 0x2502, 0x2551, }, /* ╷││║ */
+      .c[_][S][d] = { 0x256e, 0x2524, 0x2524, 0x2562, }, /* ╮┤┤╢ */
+      .c[_][S][S] = { 0x256e, 0x2524, 0x2524, 0x2562, }, /* ╮┤┤╢ */
+      .c[_][S][D] = { 0x2555, 0x2561, 0x2561, 0x2563, }, /* ╕╡╡╣ */
+      .c[_][d][_] = { 0x2577, 0x250a, 0x2502, 0x2551, }, /* ╷┊│║ */
+      .c[_][d][d] = { 0x256e, 0x2524, 0x2524, 0x2562, }, /* ╮┤┤╢ */
+      .c[_][d][S] = { 0x256e, 0x2524, 0x2524, 0x2562, }, /* ╮┤┤╢ */
+      .c[_][d][D] = { 0x2555, 0x2561, 0x2561, 0x2563, }, /* ╕╡╡╣ */
+      .c[_][D][_] = { 0x2551, 0x2551, 0x2551, 0x2551, }, /* ║║║║ */
+      .c[_][D][d] = { 0x2556, 0x2562, 0x2562, 0x2562, }, /* ╖╢╢╢ */
+      .c[_][D][S] = { 0x2556, 0x2562, 0x2562, 0x2562, }, /* ╖╢╢╢ */
+      .c[_][D][D] = { 0x2557, 0x2563, 0x2563, 0x2563, }, /* ╗╣╣╣ */
+      .c[d][_][_] = { 0x2576, 0x2570, 0x2570, 0x2559, }, /* ╶╰╰╙ */
+      .c[d][_][d] = { 0x254c, 0x2534, 0x2534, 0x2568, }, /* ╌┴┴╨ */
+      .c[d][_][S] = { 0x2500, 0x2534, 0x2534, 0x2568, }, /* ─┴┴╨ */
+      .c[d][_][D] = { 0x2550, 0x2567, 0x2567, 0x2569, }, /* ═╧╧╩ */
+      .c[d][d][_] = { 0x256d, 0x251c, 0x251c, 0x255f, }, /* ╭├├╟ */
+      .c[d][d][d] = { 0x252c, 0x002b, 0x253c, 0x256a, }, /* ┬+┼╪ */
+      .c[d][d][S] = { 0x252c, 0x253c, 0x253c, 0x256a, }, /* ┬┼┼╪ */
+      .c[d][d][D] = { 0x2564, 0x256a, 0x256a, 0x256c, }, /* ╤╪╪╬ */
+      .c[d][S][_] = { 0x256d, 0x251c, 0x251c, 0x255f, }, /* ╭├├╟ */
+      .c[d][S][d] = { 0x252c, 0x253c, 0x253c, 0x256a, }, /* ┬┼┼╪ */
+      .c[d][S][S] = { 0x252c, 0x253c, 0x253c, 0x256a, }, /* ┬┼┼╪ */
+      .c[d][S][D] = { 0x2564, 0x256a, 0x256a, 0x256c, }, /* ╤╪╪╬ */
+      .c[d][D][_] = { 0x2553, 0x255f, 0x255f, 0x255f, }, /* ╓╟╟╟ */
+      .c[d][D][d] = { 0x2565, 0x256b, 0x256b, 0x256b, }, /* ╥╫╫╫ */
+      .c[d][D][S] = { 0x2565, 0x256b, 0x256b, 0x256b, }, /* ╥╫╫╫ */
+      .c[d][D][D] = { 0x2566, 0x256c, 0x256c, 0x256c, }, /* ╦╬╬╬ */
+      .c[S][_][_] = { 0x2576, 0x2570, 0x2570, 0x2559, }, /* ╶╰╰╙ */
+      .c[S][_][d] = { 0x2500, 0x2534, 0x2534, 0x2568, }, /* ─┴┴╨ */
+      .c[S][_][S] = { 0x2500, 0x2534, 0x2534, 0x2568, }, /* ─┴┴╨ */
+      .c[S][_][D] = { 0x2550, 0x2567, 0x2567, 0x2569, }, /* ═╧╧╩ */
+      .c[S][d][_] = { 0x256d, 0x251c, 0x251c, 0x255f, }, /* ╭├├╟ */
+      .c[S][d][d] = { 0x252c, 0x253c, 0x253c, 0x256a, }, /* ┬┼┼╪ */
+      .c[S][d][S] = { 0x252c, 0x253c, 0x253c, 0x256a, }, /* ┬┼┼╪ */
+      .c[S][d][D] = { 0x2564, 0x256a, 0x256a, 0x256c, }, /* ╤╪╪╬ */
+      .c[S][S][_] = { 0x256d, 0x251c, 0x251c, 0x255f, }, /* ╭├├╟ */
+      .c[S][S][d] = { 0x252c, 0x253c, 0x253c, 0x256a, }, /* ┬┼┼╪ */
+      .c[S][S][S] = { 0x252c, 0x253c, 0x253c, 0x256a, }, /* ┬┼┼╪ */
+      .c[S][S][D] = { 0x2564, 0x256a, 0x256a, 0x256c, }, /* ╤╪╪╬ */
+      .c[S][D][_] = { 0x2553, 0x255f, 0x255f, 0x255f, }, /* ╓╟╟╟ */
+      .c[S][D][d] = { 0x2565, 0x256b, 0x256b, 0x256b, }, /* ╥╫╫╫ */
+      .c[S][D][S] = { 0x2565, 0x256b, 0x256b, 0x256b, }, /* ╥╫╫╫ */
+      .c[S][D][D] = { 0x2566, 0x256c, 0x256c, 0x256c, }, /* ╦╬╬╬ */
+      .c[D][_][_] = { 0x2550, 0x2558, 0x2558, 0x255a, }, /* ═╘╘╚ */
+      .c[D][_][d] = { 0x2550, 0x2567, 0x2567, 0x2569, }, /* ═╧╧╩ */
+      .c[D][_][S] = { 0x2550, 0x2567, 0x2567, 0x2569, }, /* ═╧╧╩ */
+      .c[D][_][D] = { 0x2550, 0x2567, 0x2567, 0x2569, }, /* ═╧╧╩ */
+      .c[D][d][_] = { 0x2552, 0x255e, 0x255e, 0x2560, }, /* ╒╞╞╠ */
+      .c[D][d][d] = { 0x2564, 0x256a, 0x256a, 0x256c, }, /* ╤╪╪╬ */
+      .c[D][d][S] = { 0x2564, 0x256a, 0x256a, 0x256c, }, /* ╤╪╪╬ */
+      .c[D][d][D] = { 0x2564, 0x256a, 0x256a, 0x256c, }, /* ╤╪╪╬ */
+      .c[D][S][_] = { 0x2552, 0x255e, 0x255e, 0x2560, }, /* ╒╞╞╠ */
+      .c[D][S][d] = { 0x2564, 0x256a, 0x256a, 0x256c, }, /* ╤╪╪╬ */
+      .c[D][S][S] = { 0x2564, 0x256a, 0x256a, 0x256c, }, /* ╤╪╪╬ */
+      .c[D][S][D] = { 0x2564, 0x256a, 0x256a, 0x256c, }, /* ╤╪╪╬ */
+      .c[D][D][_] = { 0x2554, 0x2560, 0x2560, 0x2560, }, /* ╔╠╠╠ */
+      .c[D][D][d] = { 0x2560, 0x256c, 0x256c, 0x256c, }, /* ╠╬╬╬ */
+      .c[D][D][S] = { 0x2560, 0x256c, 0x256c, 0x256c, }, /* ╠╬╬╬ */
+      .c[D][D][D] = { 0x2566, 0x256c, 0x256c, 0x256c, }, /* ╦╬╬╬ */
+    };
+  return &unicode_box;
+}
+
 static int
 ascii_line_from_render_line (int render_line)
 {
@@ -148,8 +248,10 @@ ascii_line_from_render_line (int render_line)
     case RENDER_LINE_NONE:
       return ASCII_LINE_NONE;
 
-    case RENDER_LINE_SINGLE:
     case RENDER_LINE_DASHED:
+      return ASCII_LINE_DASHED;
+
+    case RENDER_LINE_SINGLE:
     case RENDER_LINE_THICK:
     case RENDER_LINE_THIN:
       return ASCII_LINE_SINGLE;
@@ -163,8 +265,9 @@ ascii_line_from_render_line (int render_line)
 
 }
 
-static int
-make_box_index (int left_, int right_, int top_, int bottom_)
+static ucs4_t
+box_get (const struct box_chars *box,
+         int left_, int right_, int top_, int bottom_)
 {
   bool rtl = render_direction_rtl ();
   int left = ascii_line_from_render_line (rtl ? right_ : left_);
@@ -172,11 +275,7 @@ make_box_index (int left_, int right_, int top_, int bottom_)
   int top = ascii_line_from_render_line (top_);
   int bottom = ascii_line_from_render_line (bottom_);
 
-  int idx = right;
-  idx = idx * ASCII_N_LINES + bottom;
-  idx = idx * ASCII_N_LINES + left;
-  idx = idx * ASCII_N_LINES + top;
-  return idx;
+  return box->c[right][bottom][left][top];
 }
 
 /* ASCII output driver. */
@@ -189,11 +288,9 @@ struct ascii_driver
     bool emphasis;              /* Enable bold and underline in output? */
     char *chart_file_name;      /* Name of files used for charts. */
 
-#ifdef HAVE_CAIRO
     /* Colours for charts */
     struct cell_color fg;
     struct cell_color bg;
-#endif
 
     /* How the page width is determined: */
     enum {
@@ -205,7 +302,7 @@ struct ascii_driver
 
     int min_hbreak;             /* Min cell size to break across pages. */
 
-    const ucs4_t *box;          /* Line & box drawing characters. */
+    const struct box_chars *box; /* Line & box drawing characters. */
 
     /* Internal state. */
     struct file_handle *handle;
@@ -213,8 +310,9 @@ struct ascii_driver
     bool error;                 /* Output error? */
     struct u8_line *lines;      /* Page content. */
     int allocated_lines;        /* Number of lines allocated. */
-    int chart_cnt;              /* Number of charts so far. */
-    int object_cnt;             /* Number of objects so far. */
+    int n_charts;               /* Number of charts so far. */
+    int n_objects;              /* Number of objects so far. */
+    const struct pivot_table *pt;
     struct render_params params;
   };
 
@@ -235,7 +333,7 @@ static void ascii_measure_cell_width (void *, const struct table_cell *,
 static int ascii_measure_cell_height (void *, const struct table_cell *,
                                       int width);
 static void ascii_draw_cell (void *, const struct table_cell *, int color_idx,
-                             int bb[TABLE_N_AXES][2],
+                             int bb[TABLE_N_AXES][2], int valign_offset,
                              int spill[TABLE_N_AXES][2],
                              int clip[TABLE_N_AXES][2]);
 
@@ -272,9 +370,7 @@ ascii_create (struct  file_handle *fh, enum settings_output_devices device_type,
 {
   enum { BOX_ASCII, BOX_UNICODE } box;
   struct output_driver *d;
-  struct ascii_driver *a;
-
-  a = xzalloc (sizeof *a);
+  struct ascii_driver *a = XZALLOC (struct ascii_driver);
   d = &a->driver;
   output_driver_init (&a->driver, &ascii_driver_class, fh_get_file_name (fh), device_type);
   a->append = parse_boolean (opt (d, o, "append", "false"));
@@ -291,10 +387,8 @@ ascii_create (struct  file_handle *fh, enum settings_output_devices device_type,
                    : VIEW_WIDTH);
   a->min_hbreak = parse_int (opt (d, o, "min-hbreak", "-1"), -1, INT_MAX);
 
-#ifdef HAVE_CAIRO
-  parse_color (d, o, "background-color", "#FFFFFFFFFFFF", &a->bg);
-  parse_color (d, o, "foreground-color", "#000000000000", &a->fg);
-#endif
+  a->bg = parse_color (opt (d, o, "background-color", "#FFFFFFFFFFFF"));
+  a->fg = parse_color (opt (d, o, "foreground-color", "#000000000000"));
 
   const char *default_box = (terminal && (!strcmp (locale_charset (), "UTF-8")
                                           || term_is_utf8_xterm ())
@@ -303,33 +397,41 @@ ascii_create (struct  file_handle *fh, enum settings_output_devices device_type,
                     "ascii", BOX_ASCII,
                     "unicode", BOX_UNICODE,
                     NULL_SENTINEL);
-  a->box = box == BOX_ASCII ? ascii_box_chars : unicode_box_chars;
+  a->box = box == BOX_ASCII ? get_ascii_box () : get_unicode_box ();
 
   a->file = NULL;
   a->error = false;
   a->lines = NULL;
   a->allocated_lines = 0;
-  a->chart_cnt = 0;
-  a->object_cnt = 0;
+  a->n_charts = 0;
+  a->n_objects = 0;
 
-  a->params.draw_line = ascii_draw_line;
-  a->params.measure_cell_width = ascii_measure_cell_width;
-  a->params.measure_cell_height = ascii_measure_cell_height;
-  a->params.adjust_break = NULL;
-  a->params.draw_cell = ascii_draw_cell;
+  static const struct render_ops ascii_render_ops = {
+    .draw_line = ascii_draw_line,
+    .measure_cell_width = ascii_measure_cell_width,
+    .measure_cell_height = ascii_measure_cell_height,
+    .adjust_break = NULL,
+    .draw_cell = ascii_draw_cell,
+  };
+  a->params.ops = &ascii_render_ops;
   a->params.aux = a;
   a->params.size[H] = a->width;
   a->params.size[V] = INT_MAX;
   a->params.font_size[H] = 1;
   a->params.font_size[V] = 1;
-  for (int i = 0; i < RENDER_N_LINES; i++)
-    {
-      int width = i == RENDER_LINE_NONE ? 0 : 1;
-      a->params.line_widths[H][i] = width;
-      a->params.line_widths[V][i] = width;
-    }
+
+  static const int ascii_line_widths[RENDER_N_LINES] = {
+    [RENDER_LINE_NONE] = 0,
+    [RENDER_LINE_SINGLE] = 1,
+    [RENDER_LINE_DASHED] = 1,
+    [RENDER_LINE_THICK] = 1,
+    [RENDER_LINE_THIN] = 1,
+    [RENDER_LINE_DOUBLE] = 1,
+  };
+  a->params.line_widths = ascii_line_widths;
   a->params.supports_margins = false;
   a->params.rtl = render_direction_rtl ();
+  a->params.printing = true;
 
   if (!update_page_size (a, true))
     goto error;
@@ -445,89 +547,106 @@ ascii_output_lines (struct ascii_driver *a, size_t n_lines)
 
 static void
 ascii_output_table_item (struct ascii_driver *a,
-                         const struct table_item *table_item)
+                         const struct output_item *item)
 {
-  struct render_pager *p;
-
   update_page_size (a, false);
+  a->pt = item->table;
 
-  if (a->object_cnt++)
-    putc ('\n', a->file);
-
-  p = render_pager_create (&a->params, table_item);
-  for (int i = 0; render_pager_has_next (p); i++)
+  size_t *layer_indexes;
+  PIVOT_OUTPUT_FOR_EACH_LAYER (layer_indexes, item->table, true)
     {
-      if (i)
-        putc ('\n', a->file);
-      ascii_output_lines (a, render_pager_draw_next (p, INT_MAX));
+      struct render_pager *p = render_pager_create (&a->params, item->table,
+                                                    layer_indexes);
+      for (int i = 0; render_pager_has_next (p); i++)
+        {
+          if (a->n_objects++)
+            putc ('\n', a->file);
+
+          ascii_output_lines (a, render_pager_draw_next (p, INT_MAX));
+        }
+      render_pager_destroy (p);
     }
-  render_pager_destroy (p);
+
+  a->pt = NULL;
 }
 
 static void
 ascii_output_table_item_unref (struct ascii_driver *a,
-                               struct table_item *table_item)
+                               struct output_item *table_item)
 {
   ascii_output_table_item (a, table_item);
-  table_item_unref (table_item);
+  output_item_unref (table_item);
 }
 
 static void
-ascii_output_text (struct ascii_driver *a, const char *text)
-{
-  ascii_output_table_item_unref (
-    a, table_item_create (table_from_string (text), NULL, NULL));
-}
-
-static void
-ascii_submit (struct output_driver *driver,
-              const struct output_item *output_item)
+ascii_submit (struct output_driver *driver, const struct output_item *item)
 {
   struct ascii_driver *a = ascii_driver_cast (driver);
-
   if (a->error)
     return;
 
-  if (is_table_item (output_item))
-    ascii_output_table_item (a, to_table_item (output_item));
-#ifdef HAVE_CAIRO
-  else if (is_chart_item (output_item) && a->chart_file_name != NULL)
+  switch (item->type)
     {
-      struct chart_item *chart_item = to_chart_item (output_item);
-      char *file_name;
+    case OUTPUT_ITEM_TABLE:
+      ascii_output_table_item (a, item);
+      break;
 
-      file_name = xr_draw_png_chart (chart_item, a->chart_file_name,
-                                     ++a->chart_cnt,
-				     &a->fg,
-				     &a->bg);
-      if (file_name != NULL)
+    case OUTPUT_ITEM_IMAGE:
+      if (a->chart_file_name != NULL)
         {
-          struct text_item *text_item;
+          char *file_name = xr_write_png_image (
+            item->image, a->chart_file_name, ++a->n_charts);
+          if (file_name != NULL)
+            {
+              struct output_item *text_item = text_item_create_nocopy (
+                TEXT_ITEM_LOG,
+                xasprintf (_("See %s for an image."), file_name),
+                NULL);
 
-          text_item = text_item_create_format (
-            TEXT_ITEM_LOG, _("See %s for a chart."), file_name);
-
-          ascii_submit (driver, &text_item->output_item);
-          text_item_unref (text_item);
-          free (file_name);
+              ascii_submit (driver, text_item);
+              output_item_unref (text_item);
+              free (file_name);
+            }
         }
-    }
-#endif  /* HAVE_CAIRO */
-  else if (is_text_item (output_item))
-    {
-      const struct text_item *text_item = to_text_item (output_item);
-      enum text_item_type type = text_item_get_type (text_item);
+      break;
 
-      if (type != TEXT_ITEM_PAGE_TITLE && type != TEXT_ITEM_EJECT_PAGE)
+    case OUTPUT_ITEM_CHART:
+      if (a->chart_file_name != NULL)
+        {
+          char *file_name = xr_draw_png_chart (
+            item->chart, a->chart_file_name, ++a->n_charts, &a->fg, &a->bg);
+          if (file_name != NULL)
+            {
+              struct output_item *text_item = text_item_create_nocopy (
+                TEXT_ITEM_LOG,
+                xasprintf (_("See %s for a chart."), file_name),
+                NULL);
+
+              ascii_submit (driver, text_item);
+              output_item_unref (text_item);
+              free (file_name);
+            }
+        }
+      break;
+
+    case OUTPUT_ITEM_TEXT:
+      if (item->text.subtype != TEXT_ITEM_PAGE_TITLE)
         ascii_output_table_item_unref (
-          a, text_item_to_table_item (text_item_ref (text_item)));
-    }
-  else if (is_message_item (output_item))
-    {
-      const struct message_item *message_item = to_message_item (output_item);
-      char *s = msg_to_string (message_item_get_msg (message_item));
-      ascii_output_text (a, s);
-      free (s);
+          a, text_item_to_table_item (output_item_ref (item)));
+      break;
+
+    case OUTPUT_ITEM_MESSAGE:
+      ascii_output_table_item_unref (
+        a, text_item_to_table_item (
+          message_item_to_text_item (
+            output_item_ref (item))));
+      break;
+
+    case OUTPUT_ITEM_GROUP:
+      break;
+
+    case OUTPUT_ITEM_PAGE_BREAK:
+      break;
     }
 }
 
@@ -538,10 +657,10 @@ const struct output_driver_factory list_driver_factory =
 
 static const struct output_driver_class ascii_driver_class =
   {
-    "text",
-    ascii_destroy,
-    ascii_submit,
-    ascii_flush,
+    .name = "text",
+    .destroy = ascii_destroy,
+    .submit = ascii_submit,
+    .flush = ascii_flush,
   };
 
 static char *ascii_reserve (struct ascii_driver *, int y, int x0, int x1,
@@ -573,8 +692,7 @@ ascii_draw_line (void *a_, int bb[TABLE_N_AXES][2],
     return;
 
   /* Draw. */
-  uc = a->box[make_box_index (styles[V][0], styles[V][1],
-                              styles[H][0], styles[H][1])];
+  uc = box_get (a->box, styles[V][0], styles[V][1], styles[H][0], styles[H][1]);
   mblen = u8_uctomb (CHAR_CAST (uint8_t *, mbchar), uc, 6);
   for (y = y0; y < y1; y++)
     {
@@ -603,14 +721,8 @@ ascii_measure_cell_width (void *a_, const struct table_cell *cell,
   clip[H][0] = clip[H][1] = clip[V][0] = clip[V][1] = 0;
   ascii_layout_cell (a, cell, bb, clip, max_width, &h);
 
-  if (cell->n_footnotes || strchr (cell->text, ' ')
-      || cell->n_subscripts || cell->superscript)
-    {
-      bb[H][1] = 1;
-      ascii_layout_cell (a, cell, bb, clip, min_width, &h);
-    }
-  else
-    *min_width = *max_width;
+  bb[H][1] = 1;
+  ascii_layout_cell (a, cell, bb, clip, min_width, &h);
 }
 
 static int
@@ -632,13 +744,14 @@ ascii_measure_cell_height (void *a_, const struct table_cell *cell, int width)
 
 static void
 ascii_draw_cell (void *a_, const struct table_cell *cell, int color_idx UNUSED,
-                 int bb[TABLE_N_AXES][2],
+                 int bb[TABLE_N_AXES][2], int valign_offset,
                  int spill[TABLE_N_AXES][2] UNUSED,
                  int clip[TABLE_N_AXES][2])
 {
   struct ascii_driver *a = a_;
   int w, h;
 
+  bb[V][0] += valign_offset;
   ascii_layout_cell (a, cell, bb, clip, &w, &h);
 }
 
@@ -659,7 +772,7 @@ ascii_reserve (struct ascii_driver *a, int y, int x0, int x1, int n)
 }
 
 static void
-text_draw (struct ascii_driver *a, enum table_halign halign, int options,
+text_draw (struct ascii_driver *a, enum table_halign halign, bool numeric,
            bool bold, bool underline,
            int bb[TABLE_N_AXES][2], int clip[TABLE_N_AXES][2],
            int y, const uint8_t *string, int n, size_t width)
@@ -673,7 +786,7 @@ text_draw (struct ascii_driver *a, enum table_halign halign, int options,
   if (y < y0 || y >= y1)
     return;
 
-  switch (table_halign_interpret (halign, options & TAB_NUMERIC))
+  switch (table_halign_interpret (halign, numeric))
     {
     case TABLE_HALIGN_LEFT:
       x = bb[H][0];
@@ -797,20 +910,6 @@ text_draw (struct ascii_driver *a, enum table_halign halign, int options,
     }
 }
 
-static char *
-add_markers (const char *text, const struct table_cell *cell)
-{
-  struct string s = DS_EMPTY_INITIALIZER;
-  ds_put_cstr (&s, text);
-  for (size_t i = 0; i < cell->n_subscripts; i++)
-    ds_put_format (&s, "%c%s", i ? ',' : '_', cell->subscripts[i]);
-  if (cell->superscript)
-    ds_put_format (&s, "^%s", cell->superscript);
-  for (size_t i = 0; i < cell->n_footnotes; i++)
-    ds_put_format (&s, "[%s]", cell->footnotes[i]->marker);
-  return ds_steal_cstr (&s);
-}
-
 static void
 ascii_layout_cell (struct ascii_driver *a, const struct table_cell *cell,
                    int bb[TABLE_N_AXES][2], int clip[TABLE_N_AXES][2],
@@ -819,34 +918,21 @@ ascii_layout_cell (struct ascii_driver *a, const struct table_cell *cell,
   *widthp = 0;
   *heightp = 0;
 
-  /* Get the basic textual contents. */
-  const char *plain_text = (cell->options & TAB_MARKUP
-                            ? output_get_text_from_markup (cell->text)
-                            : cell->text);
-
-  /* Append footnotes, subscripts, superscript if any. */
-  const char *text;
-  if (cell->n_footnotes || cell->n_subscripts || cell->superscript)
-    {
-      text = add_markers (plain_text, cell);
-      if (plain_text != cell->text)
-        free (CONST_CAST (char *, plain_text));
-    }
-  else
-    text = plain_text;
+  struct string body = DS_EMPTY_INITIALIZER;
+  bool numeric = pivot_value_format (cell->value, a->pt, &body);
 
   /* Calculate length; if it's zero, then there's nothing to do. */
-  size_t length = strlen (text);
-  if (!length)
+  if (ds_is_empty (&body))
     {
-      if (text != cell->text)
-        free (CONST_CAST (char *, text));
+      ds_destroy (&body);
       return;
     }
 
+  size_t length = ds_length (&body);
+  const uint8_t *text = CHAR_CAST (uint8_t *, ds_cstr (&body));
+
   char *breaks = xmalloc (length + 1);
-  u8_possible_linebreaks (CHAR_CAST (const uint8_t *, text), length,
-                          "UTF-8", breaks);
+  u8_possible_linebreaks (text, length, "UTF-8", breaks);
   breaks[length] = (breaks[length - 1] == UC_BREAK_MANDATORY
                     ? UC_BREAK_PROHIBITED : UC_BREAK_POSSIBLE);
 
@@ -854,7 +940,7 @@ ascii_layout_cell (struct ascii_driver *a, const struct table_cell *cell,
   int bb_width = bb[H][1] - bb[H][0];
   for (int y = bb[V][0]; y < bb[V][1] && pos < length; y++)
     {
-      const uint8_t *line = CHAR_CAST (const uint8_t *, text + pos);
+      const uint8_t *line = text + pos;
       const char *b = breaks + pos;
       size_t n = length - pos;
 
@@ -905,9 +991,9 @@ ascii_layout_cell (struct ascii_driver *a, const struct table_cell *cell,
       width -= ofs - graph_ofs;
 
       /* Draw text. */
-      text_draw (a, cell->style->cell_style.halign, cell->options,
-                 cell->style->font_style.bold,
-                 cell->style->font_style.underline,
+      text_draw (a, cell->cell_style->halign, numeric,
+                 cell->font_style->bold,
+                 cell->font_style->underline,
                  bb, clip, y, line, graph_ofs, width);
 
       /* If a new-line ended the line, just skip the new-line.  Otherwise, skip
@@ -925,8 +1011,7 @@ ascii_layout_cell (struct ascii_driver *a, const struct table_cell *cell,
     }
 
   free (breaks);
-  if (text != cell->text)
-    free (CONST_CAST (char *, text));
+  ds_destroy (&body);
 }
 
 void
@@ -940,14 +1025,24 @@ ascii_test_write (struct output_driver *driver,
   if (!a->file)
     return;
 
-  struct area_style style = {
-    .cell_style.halign = TABLE_HALIGN_LEFT,
-    .font_style.bold = bold,
-    .font_style.underline = underline,
+  struct cell_style cell_style = { .halign = TABLE_HALIGN_LEFT };
+  struct font_style font_style = {
+    .bold = bold,
+    .underline = underline,
+  };
+  const struct pivot_value value = {
+    .text = {
+      .type = PIVOT_VALUE_TEXT,
+      .local = CONST_CAST (char *, s),
+      .c = CONST_CAST (char *, s),
+      .id = CONST_CAST (char *, s),
+      .user_provided = true,
+    },
   };
   struct table_cell cell = {
-    .text = CONST_CAST (char *, s),
-    .style = &style,
+    .value = &value,
+    .font_style = &font_style,
+    .cell_style = &cell_style,
   };
 
   bb[TABLE_HORZ][0] = x;
@@ -955,7 +1050,13 @@ ascii_test_write (struct output_driver *driver,
   bb[TABLE_VERT][0] = y;
   bb[TABLE_VERT][1] = INT_MAX;
 
+  struct pivot_table pt = {
+    .show_values = SETTINGS_VALUE_SHOW_DEFAULT,
+    .show_variables = SETTINGS_VALUE_SHOW_DEFAULT,
+  };
+  a->pt = &pt;
   ascii_layout_cell (a, &cell, bb, bb, &width, &height);
+  a->pt = NULL;
 }
 
 void
