@@ -21,9 +21,8 @@
 
 #include <ctype.h>
 #include <errno.h>
-#include <libxml/parser.h>
-#include <libxml/tree.h>
 #include <limits.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -38,14 +37,12 @@
 #include "libpspp/string-map.h"
 #include "libpspp/string-set.h"
 #include "libpspp/str.h"
-#include "output/group-item.h"
-#include "output/message-item.h"
 #include "output/output-item.h"
-#include "output/text-item.h"
 
 #include "gl/error.h"
 #include "gl/xalloc.h"
 #include "gl/xmemdup0.h"
+#include "gl/xvasprintf.h"
 
 #include "gettext.h"
 #define _(msgid) gettext (msgid)
@@ -54,16 +51,12 @@ struct output_engine
   {
     struct ll ll;                  /* Node for this engine. */
     struct llx_list drivers;       /* Contains "struct output_driver"s. */
-    struct string deferred_text;   /* Output text being accumulated. */
-    enum text_item_type deferred_type; /* Type of text being accumulated. */
+    struct output_item *deferred_text; /* Output text being accumulated. */
     char *command_name;            /* Name of command being processed. */
     char *title, *subtitle;        /* Components of page title. */
 
-    /* Output grouping stack.
-
-       TEXT_ITEM_GROUP_OPEN pushes a group on the stack and
-       TEXT_ITEM_GROUP_CLOSE pops one off. */
-    char **groups;               /* Command names of nested sections. */
+    /* Output grouping stack. */
+    struct output_item **groups;
     size_t n_groups;
     size_t allocated_groups;
 
@@ -98,10 +91,9 @@ put_strftime (const char *key, const char *format,
 void
 output_engine_push (void)
 {
-  struct output_engine *e = xzalloc (sizeof (*e));
+  struct output_engine *e = XZALLOC (struct output_engine);
 
   llx_init (&e->drivers);
-  ds_init_empty (&e->deferred_text);
 
   string_map_init (&e->heading_vars);
 
@@ -117,19 +109,17 @@ void
 output_engine_pop (void)
 {
   struct ll *head = ll_pop_head (&engine_stack);
-  struct output_engine *e =ll_data (head, struct output_engine, ll);
+  struct output_engine *e = ll_data (head, struct output_engine, ll);
 
-  while (!llx_is_empty (&e->drivers))
-    {
-      struct output_driver *d = llx_pop_head (&e->drivers, &llx_malloc_mgr);
-      output_driver_destroy (d);
-    }
-  ds_destroy (&e->deferred_text);
+  struct output_driver *d;
+  llx_for_each_preremove (d, &e->drivers, &llx_malloc_mgr)
+    output_driver_destroy (d);
+  output_item_unref (e->deferred_text);
   free (e->command_name);
   free (e->title);
   free (e->subtitle);
-  for (size_t i = 0; i < e->n_groups; i++)
-    free (e->groups[i]);
+  if (e->n_groups)
+    output_item_unref (e->groups[0]);
   free (e->groups);
   string_map_destroy (&e->heading_vars);
   free (e);
@@ -144,34 +134,84 @@ output_get_supported_formats (struct string_set *formats)
     string_set_insert (formats, (*fp)->extension);
 }
 
+static bool
+output_driver_should_show (const struct output_driver *d,
+                           const struct output_item *item)
+{
+  enum settings_output_type type = SETTINGS_OUTPUT_RESULT;
+  switch (item->type)
+    {
+    case OUTPUT_ITEM_MESSAGE:
+      type = (item->message->severity == MSG_S_NOTE
+              ? SETTINGS_OUTPUT_NOTE
+              : SETTINGS_OUTPUT_ERROR);
+      break;
+
+    case OUTPUT_ITEM_TEXT:
+      if (item->text.subtype == TEXT_ITEM_SYNTAX)
+        type = SETTINGS_OUTPUT_SYNTAX;
+      break;
+
+    case OUTPUT_ITEM_CHART:
+    case OUTPUT_ITEM_GROUP:
+    case OUTPUT_ITEM_IMAGE:
+    case OUTPUT_ITEM_PAGE_BREAK:
+    case OUTPUT_ITEM_TABLE:
+      break;
+    }
+
+  return (settings_get_output_routing (type) & d->device_type) != 0;
+}
+
+/* Adds to OUT the subset of IN that driver D should show, considering routing
+   and visibility of each item, and flattening groups for drivers that don't
+   handle them internally. */
+static void
+make_driver_output_subset (const struct output_item *in,
+                           const struct output_driver *d,
+                           struct output_item *out)
+{
+  if (in->type == OUTPUT_ITEM_GROUP)
+    {
+      /* If we should include the group itself, then clone IN inside OUT, and
+         add any children to the clone instead to OUT directly. */
+      if (output_driver_should_show (d, in) && d->class->handles_groups)
+        {
+          struct output_item *group = group_item_clone_empty (in);
+          group_item_add_child (out, group);
+          out = group;
+        }
+
+      for (size_t i = 0; i < in->group.n_children; i++)
+        make_driver_output_subset (in->group.children[i], d, out);
+    }
+  else
+    {
+      if (output_driver_should_show (d, in)
+          && (in->show || d->class->handles_show))
+        group_item_add_child (out, output_item_ref (in));
+    }
+}
+
 static void
 output_submit__ (struct output_engine *e, struct output_item *item)
 {
-  struct llx *llx, *next;
+  if (e->n_groups > 0)
+    {
+      group_item_add_child (e->groups[e->n_groups - 1], item);
+      return;
+    }
 
-  for (llx = llx_head (&e->drivers); llx != llx_null (&e->drivers); llx = next)
+  struct llx *llx, *next;
+  llx_for_each_safe (llx, next, &e->drivers)
     {
       struct output_driver *d = llx_data (llx);
-      enum settings_output_type type;
 
-      next = llx_next (llx);
-
-      if (is_message_item (item))
-        {
-          const struct msg *m = message_item_get_msg (to_message_item (item));
-          if (m->severity == MSG_S_NOTE)
-            type = SETTINGS_OUTPUT_NOTE;
-          else
-            type = SETTINGS_OUTPUT_ERROR;
-        }
-      else if (is_text_item (item)
-               && text_item_get_type (to_text_item (item)) == TEXT_ITEM_SYNTAX)
-        type = SETTINGS_OUTPUT_SYNTAX;
-      else
-        type = SETTINGS_OUTPUT_RESULT;
-
-      if (settings_get_output_routing (type) & d->device_type)
-        d->class->submit (d, item);
+      struct output_item *root = root_item_create ();
+      make_driver_output_subset (item, d, root);
+      for (size_t i = 0; i < root->group.n_children; i++)
+        d->class->submit (d, root->group.children[i]);
+      output_item_unref (root);
     }
 
   output_item_unref (item);
@@ -180,40 +220,29 @@ output_submit__ (struct output_engine *e, struct output_item *item)
 static void
 flush_deferred_text (struct output_engine *e)
 {
-  if (!ds_is_empty (&e->deferred_text))
+  struct output_item *deferred_text = e->deferred_text;
+  if (deferred_text)
     {
-      char *text = ds_steal_cstr (&e->deferred_text);
-      output_submit__ (e, text_item_super (text_item_create_nocopy (
-                                             e->deferred_type, text)));
+      e->deferred_text = NULL;
+      output_submit__ (e, deferred_text);
     }
 }
 
 static bool
 defer_text (struct output_engine *e, struct output_item *item)
 {
-  if (!is_text_item (item))
+  if (item->type != OUTPUT_ITEM_TEXT)
     return false;
 
-  struct text_item *text_item = to_text_item (item);
-  if (text_item->markup)        /* XXX */
-    return false;
-
-  enum text_item_type type = text_item_get_type (text_item);
-  if (type != TEXT_ITEM_SYNTAX && type != TEXT_ITEM_LOG)
-    return false;
-
-  if (!ds_is_empty (&e->deferred_text) && e->deferred_type != type)
-    flush_deferred_text (e);
-
-  e->deferred_type = type;
-
-  if (!ds_is_empty (&e->deferred_text))
-    ds_put_byte (&e->deferred_text, '\n');
-
-  const char *text = text_item_get_text (text_item);
-  ds_put_cstr (&e->deferred_text, text);
-  output_item_unref (item);
-
+  if (!e->deferred_text)
+    e->deferred_text = output_item_unshare (item);
+  else if (text_item_append (e->deferred_text, item))
+    output_item_unref (item);
+  else
+    {
+      flush_deferred_text (e);
+      e->deferred_text = output_item_unshare (item);
+    }
   return true;
 }
 
@@ -234,55 +263,14 @@ output_submit (struct output_item *item)
     return;
   flush_deferred_text (e);
 
-  if (is_group_open_item (item))
-    {
-      const struct group_open_item *group_open_item
-        = to_group_open_item (item);
-      if (e->n_groups >= e->allocated_groups)
-        e->groups = x2nrealloc (e->groups, &e->allocated_groups,
-                                sizeof *e->groups);
-      e->groups[e->n_groups] = (group_open_item->command_name
-                                ? xstrdup (group_open_item->command_name)
-                                : NULL);
-      e->n_groups++;
-    }
-  else if (is_group_close_item (item))
-    {
-      assert (e->n_groups > 0);
-
-      size_t idx = --e->n_groups;
-      free (e->groups[idx]);
-      if (idx >= 1 && idx <= 4)
-        {
-          char *key = xasprintf ("Head%zu", idx);
-          free (string_map_find_and_delete (&e->heading_vars, key));
-          free (key);
-        }
-    }
-  else if (is_text_item (item))
-    {
-      const struct text_item *text_item = to_text_item (item);
-      enum text_item_type type = text_item_get_type (text_item);
-      const char *text = text_item_get_text (text_item);
-      if (type == TEXT_ITEM_TITLE
-          && e->n_groups >= 1 && e->n_groups <= 4)
-        {
-          char *key = xasprintf ("Head%zu", e->n_groups);
-          string_map_replace (&e->heading_vars, key, text);
-          free (key);
-        }
-      else if (type == TEXT_ITEM_PAGE_TITLE)
-        string_map_replace (&e->heading_vars, "PageTitle", text);
-    }
+  /* XXX heading_vars */
 
   output_submit__ (e, item);
 }
 
-/* Returns the name of the command currently being parsed, in all uppercase.
-   The caller must free the returned value.
-
-   Returns NULL if no command is being parsed. */
-char *
+/* Returns the name of the command currently being parsed, or NULL if no
+   command is being parsed. */
+const char *
 output_get_command_name (void)
 {
   struct output_engine *e = engine_stack_top ();
@@ -290,23 +278,64 @@ output_get_command_name (void)
     return NULL;
 
   for (size_t i = e->n_groups; i-- > 0;)
-    if (e->groups[i])
-      return utf8_to_upper (e->groups[i]);
+    if (e->groups[i]->command_name)
+      return e->groups[i]->command_name;
 
   return NULL;
 }
 
+char *
+output_get_uppercase_command_name (void)
+{
+  const char *command_name = output_get_command_name ();
+  return command_name ? utf8_to_upper (command_name) : NULL;
+}
+
+size_t
+output_open_group (struct output_item *item)
+{
+  struct output_engine *e = engine_stack_top ();
+  if (e == NULL)
+    return 0;
+
+  if (e->n_groups >= e->allocated_groups)
+    e->groups = x2nrealloc (e->groups, &e->allocated_groups,
+                            sizeof *e->groups);
+  e->groups[e->n_groups++] = item;
+  if (e->n_groups > 1)
+    group_item_add_child (e->groups[e->n_groups - 2], item);
+
+  return e->n_groups - 1;
+}
+
+void
+output_close_groups (size_t nesting_level)
+{
+  struct output_engine *e = engine_stack_top ();
+  if (e == NULL)
+    return;
+
+  while (e->n_groups > nesting_level)
+    {
+      flush_deferred_text (e);
+
+      struct output_item *group = e->groups[--e->n_groups];
+      if (e->n_groups == 0)
+        output_submit__ (e, group);
+    }
+}
+
 /* Flushes output to screen devices, so that the user can see
    output that doesn't fill up an entire page. */
 void
 output_flush (void)
 {
   struct output_engine *e = engine_stack_top ();
-  struct llx *llx;
 
   flush_deferred_text (e);
-  for (llx = llx_head (&e->drivers); llx != llx_null (&e->drivers);
-       llx = llx_next (llx))
+
+  struct llx *llx;
+  llx_for_each (llx, &e->drivers)
     {
       struct output_driver *d = llx_data (llx);
       if (d->device_type & SETTINGS_DEVICE_TERMINAL && d->class->flush != NULL)
@@ -318,15 +347,38 @@ static void
 output_set_title__ (struct output_engine *e, char **dst, const char *src)
 {
   free (*dst);
-  *dst = src ? xstrdup (src) : NULL;
+  *dst = xstrdup_if_nonnull (src);
 
   char *page_title
     = (e->title && e->subtitle ? xasprintf ("%s\n%s", e->title, e->subtitle)
        : e->title ? xstrdup (e->title)
        : e->subtitle ? xstrdup (e->subtitle)
        : xzalloc (1));
-  text_item_submit (text_item_create_nocopy (TEXT_ITEM_PAGE_TITLE,
-                                             page_title));
+  output_item_submit (text_item_create_nocopy (TEXT_ITEM_PAGE_TITLE,
+                                               page_title, NULL));
+}
+
+void PRINTF_FORMAT (1, 2)
+output_log (const char *format, ...)
+{
+  va_list args;
+  va_start (args, format);
+  char *s = xvasprintf (format, args);
+  va_end (args);
+
+  output_log_nocopy (s);
+}
+
+void
+output_log_nocopy (char *s)
+{
+  output_submit (text_item_create_nocopy (TEXT_ITEM_LOG, s, NULL));
+}
+
+const char *
+output_get_title (void)
+{
+  return engine_stack_top ()->title;
 }
 
 void
@@ -337,6 +389,12 @@ output_set_title (const char *title)
   output_set_title__ (e, &e->title, title);
 }
 
+const char *
+output_get_subtitle (void)
+{
+  return engine_stack_top ()->subtitle;
+}
+
 void
 output_set_subtitle (const char *subtitle)
 {
@@ -352,14 +410,6 @@ output_set_filename (const char *filename)
 
   string_map_replace (&e->heading_vars, "Filename", filename);
 }
-
-size_t
-output_get_group_level (void)
-{
-  struct output_engine *e = engine_stack_top ();
-
-  return e->n_groups;
-}
 
 void
 output_driver_init (struct output_driver *driver,
@@ -429,18 +479,32 @@ output_driver_is_registered (const struct output_driver *driver)
 {
   return output_driver_get_engine (driver) != NULL;
 }
+
+void
+output_set_page_setup (const struct page_setup *ps)
+{
+  struct output_engine *e = engine_stack_top ();
+
+  struct llx *llx;
+  llx_for_each (llx, &e->drivers)
+    {
+      struct output_driver *d = llx_data (llx);
+      if (d->class->setup)
+        d->class->setup (d, ps);
+    }
+}
 
-extern const struct output_driver_factory txt_driver_factory;
-extern const struct output_driver_factory list_driver_factory;
-extern const struct output_driver_factory html_driver_factory;
 extern const struct output_driver_factory csv_driver_factory;
+extern const struct output_driver_factory html_driver_factory;
+extern const struct output_driver_factory list_driver_factory;
 extern const struct output_driver_factory odt_driver_factory;
-extern const struct output_driver_factory spv_driver_factory;
-#ifdef HAVE_CAIRO
 extern const struct output_driver_factory pdf_driver_factory;
+extern const struct output_driver_factory png_driver_factory;
 extern const struct output_driver_factory ps_driver_factory;
+extern const struct output_driver_factory spv_driver_factory;
 extern const struct output_driver_factory svg_driver_factory;
-#endif
+extern const struct output_driver_factory tex_driver_factory;
+extern const struct output_driver_factory txt_driver_factory;
 
 static const struct output_driver_factory *factories[] =
   {
@@ -450,11 +514,11 @@ static const struct output_driver_factory *factories[] =
     &csv_driver_factory,
     &odt_driver_factory,
     &spv_driver_factory,
-#ifdef HAVE_CAIRO
     &pdf_driver_factory,
     &ps_driver_factory,
     &svg_driver_factory,
-#endif
+    &png_driver_factory,
+    &tex_driver_factory,
     NULL
   };
 
@@ -566,30 +630,6 @@ output_driver_parse_option (const char *option, struct string_map *options)
   string_map_insert_nocopy (options, key, value);
 }
 
-/* Extracts the actual text content from the given Pango MARKUP and returns it
-   as as a malloc()'d string. */
-char *
-output_get_text_from_markup (const char *markup)
-{
-  xmlParserCtxt *parser = xmlCreatePushParserCtxt (NULL, NULL, NULL, 0, NULL);
-  if (!parser)
-    return xstrdup (markup);
-
-  xmlParseChunk (parser, "<xml>", strlen ("<xml>"), false);
-  xmlParseChunk (parser, markup, strlen (markup), false);
-  xmlParseChunk (parser, "</xml>", strlen ("</xml>"), true);
-
-  char *content
-    = (parser->wellFormed
-       ? CHAR_CAST (char *,
-                    xmlNodeGetContent (xmlDocGetRootElement (parser->myDoc)))
-       : xstrdup (markup));
-  xmlFreeDoc (parser->myDoc);
-  xmlFreeParserCtxt (parser);
-
-  return content;
-}
-
 char *
 output_driver_substitute_heading_vars (const char *src, int page_number)
 {
