@@ -38,11 +38,11 @@
 #include "libpspp/zip-writer.h"
 #include "data/file-handle-def.h"
 #include "output/driver-provider.h"
-#include "output/message-item.h"
 #include "output/options.h"
-#include "output/table-item.h"
+#include "output/output-item.h"
+#include "output/pivot-table.h"
+#include "output/pivot-output.h"
 #include "output/table-provider.h"
-#include "output/text-item.h"
 
 #include "gl/xalloc.h"
 
@@ -51,6 +51,10 @@
 
 #define _xml(X) (CHAR_CAST (const xmlChar *, X))
 
+/* This file uses TABLE_HORZ and TABLE_VERT enough to warrant abbreviating. */
+#define H TABLE_HORZ
+#define V TABLE_VERT
+
 struct odt_driver
 {
   struct output_driver driver;
@@ -190,6 +194,18 @@ write_style_data (struct odt_driver *odt)
     xmlTextWriterEndElement (w); /* style:style */
   }
 
+  {
+    xmlTextWriterStartElement (w, _xml ("style:style"));
+    xmlTextWriterWriteAttribute (w, _xml ("style:name"), _xml ("superscript"));
+    xmlTextWriterWriteAttribute (w, _xml ("style:family"), _xml ("text"));
+
+    xmlTextWriterStartElement (w, _xml ("style:text-properties"));
+    xmlTextWriterWriteAttribute (w, _xml ("style:text-position"),
+                                 _xml ("super 58%"));
+    xmlTextWriterEndElement (w); /* style:text-properties */
+
+    xmlTextWriterEndElement (w); /* style:style */
+  }
 
   xmlTextWriterEndElement (w); /* office:styles */
   xmlTextWriterEndElement (w); /* office:document-styles */
@@ -269,7 +285,6 @@ odt_create (struct file_handle *fh, enum settings_output_devices device_type,
             struct string_map *o UNUSED)
 {
   struct output_driver *d;
-  struct odt_driver *odt;
   struct zip_writer *zip;
   const char *file_name = fh_get_file_name (fh);
 
@@ -277,7 +292,7 @@ odt_create (struct file_handle *fh, enum settings_output_devices device_type,
   if (zip == NULL)
     return NULL;
 
-  odt = xzalloc (sizeof *odt);
+  struct odt_driver *odt = XZALLOC (struct odt_driver);
   d = &odt->driver;
 
   output_driver_init (d, &odt_driver_class, file_name, device_type);
@@ -398,75 +413,74 @@ write_xml_with_line_breaks (struct odt_driver *odt, const char *line_)
 }
 
 static void
-write_footnote (struct odt_driver *odt, const struct footnote *f)
+write_footnotes (struct odt_driver *odt,
+                 const struct pivot_table *pt,
+                 const size_t *footnote_indexes,
+                 size_t n_footnotes)
 {
-  xmlTextWriterStartElement (odt->content_wtr, _xml("text:note"));
-  xmlTextWriterWriteAttribute (odt->content_wtr, _xml("text:note-class"),
-                               _xml("footnote"));
-
-  xmlTextWriterStartElement (odt->content_wtr, _xml("text:note-citation"));
-  if (strlen (f->marker) > 1)
-    xmlTextWriterWriteFormatAttribute (odt->content_wtr, _xml("text:label"),
-                                       "(%s)", f->marker);
-  else
-    xmlTextWriterWriteAttribute (odt->content_wtr, _xml("text:label"),
-                                 _xml(f->marker));
-  xmlTextWriterEndElement (odt->content_wtr);
-
-  xmlTextWriterStartElement (odt->content_wtr, _xml("text:note-body"));
-  xmlTextWriterStartElement (odt->content_wtr, _xml("text:p"));
-  write_xml_with_line_breaks (odt, f->content);
-  xmlTextWriterEndElement (odt->content_wtr);
-  xmlTextWriterEndElement (odt->content_wtr);
-
-  xmlTextWriterEndElement (odt->content_wtr);
-}
-
-static void
-write_table_item_text (struct odt_driver *odt,
-                       const struct table_item_text *text)
-{
-  if (!text)
-    return;
-
-  xmlTextWriterStartElement (odt->content_wtr, _xml("text:h"));
-  xmlTextWriterWriteFormatAttribute (odt->content_wtr,
-                                     _xml("text:outline-level"), "%d", 2);
-  xmlTextWriterWriteString (odt->content_wtr, _xml (text->content));
-  for (size_t i = 0; i < text->n_footnotes; i++)
-    write_footnote (odt, text->footnotes[i]);
-  xmlTextWriterEndElement (odt->content_wtr);
-}
-
-static void
-write_table_item_layers (struct odt_driver *odt,
-                         const struct table_item_layers *layers)
-{
-  if (!layers)
-    return;
-
-  for (size_t i = 0; i < layers->n_layers; i++)
+  for (size_t i = 0; i < n_footnotes; i++)
     {
-      const struct table_item_layer *layer = &layers->layers[i];
-      xmlTextWriterStartElement (odt->content_wtr, _xml("text:h"));
-      xmlTextWriterWriteFormatAttribute (odt->content_wtr,
-                                         _xml("text:outline-level"), "%d", 2);
-      xmlTextWriterWriteString (odt->content_wtr, _xml (layer->content));
-      for (size_t i = 0; i < layer->n_footnotes; i++)
-        write_footnote (odt, layer->footnotes[i]);
-      xmlTextWriterEndElement (odt->content_wtr);
+      const struct pivot_footnote *f = pt->footnotes[footnote_indexes[i]];
+      if (f->show)
+        {
+          xmlTextWriterStartElement (odt->content_wtr, _xml("text:span"));
+          xmlTextWriterWriteAttribute (odt->content_wtr,
+                                       _xml("text:style-name"),
+                                       _xml("superscript"));
+          char *s = pivot_footnote_marker_string (f, pt);
+          write_xml_with_line_breaks (odt, s);
+          free (s);
+          xmlTextWriterEndElement (odt->content_wtr);
+        }
     }
 }
 
 static void
-write_table (struct odt_driver *odt, const struct table_item *item)
+write_table_item_cell (struct odt_driver *odt,
+                       const struct pivot_table *pt,
+                       const struct table_cell *cell)
 {
-  const struct table *tab = table_item_get_table (item);
-  int r, c;
+  struct string body = DS_EMPTY_INITIALIZER;
+  pivot_value_format_body (cell->value, pt, &body);
+  xmlTextWriterWriteString (odt->content_wtr, _xml (ds_cstr (&body)));
+  ds_destroy (&body);
+
+  const struct pivot_value_ex *ex = pivot_value_ex (cell->value);
+  write_footnotes (odt, pt, ex->footnote_indexes, ex->n_footnotes);
+}
+
+static void
+write_table__ (struct odt_driver *odt, const struct pivot_table *pt,
+               const struct table *t)
+{
+  if (t)
+    {
+      for (size_t y = 0; y < t->n[V]; y++)
+        {
+          xmlTextWriterStartElement (odt->content_wtr, _xml("text:h"));
+          xmlTextWriterWriteFormatAttribute (odt->content_wtr,
+                                             _xml("text:outline-level"), "%d", 2);
+
+          struct table_cell cell;
+          table_get_cell (t, 0, y, &cell);
+          write_table_item_cell (odt, pt, &cell);
+
+          xmlTextWriterEndElement (odt->content_wtr);
+        }
+    }
+}
+
+static void
+write_table_layer (struct odt_driver *odt, const struct pivot_table *pt,
+                   const size_t *layer_indexes)
+{
+  struct table *title, *layers, *body, *caption, *footnotes;
+  pivot_output (pt, layer_indexes, true, &title, &layers, &body,
+                &caption, &footnotes, NULL, NULL);
 
   /* Write a heading for the table */
-  write_table_item_text (odt, table_item_get_title (item));
-  write_table_item_layers (odt, table_item_get_layers (item));
+  write_table__ (odt, pt, title);
+  write_table__ (odt, pt, layers);
 
   /* Start table */
   xmlTextWriterStartElement (odt->content_wtr, _xml("table:table"));
@@ -476,29 +490,29 @@ write_table (struct odt_driver *odt, const struct table_item *item)
 
   /* Start column definitions */
   xmlTextWriterStartElement (odt->content_wtr, _xml("table:table-column"));
-  xmlTextWriterWriteFormatAttribute (odt->content_wtr, _xml("table:number-columns-repeated"), "%d", table_nc (tab));
+  xmlTextWriterWriteFormatAttribute (odt->content_wtr, _xml("table:number-columns-repeated"), "%d", body->n[H]);
   xmlTextWriterEndElement (odt->content_wtr);
 
 
   /* Deal with row headers */
-  if (table_ht (tab) > 0)
+  if (body->h[V][0] > 0)
     xmlTextWriterStartElement (odt->content_wtr, _xml("table:table-header-rows"));
 
 
   /* Write all the rows */
-  for (r = 0 ; r < table_nr (tab); ++r)
+  for (int r = 0 ; r < body->n[V]; ++r)
     {
       /* Start row definition */
       xmlTextWriterStartElement (odt->content_wtr, _xml("table:table-row"));
 
       /* Write all the columns */
-      for (c = 0 ; c < table_nc (tab) ; ++c)
+      for (int c = 0 ; c < body->n[H] ; ++c)
 	{
           struct table_cell cell;
 
-          table_get_cell (tab, c, r, &cell);
+          table_get_cell (body, c, r, &cell);
 
-          if (c == cell.d[TABLE_HORZ][0] && r == cell.d[TABLE_VERT][0])
+          if (c == cell.d[H][0] && r == cell.d[V][0])
             {
               int colspan = table_cell_colspan (&cell);
               int rowspan = table_cell_rowspan (&cell);
@@ -518,23 +532,12 @@ write_table (struct odt_driver *odt, const struct table_item *item)
 
               xmlTextWriterStartElement (odt->content_wtr, _xml("text:p"));
 
-              if (r < table_ht (tab) || c < table_hl (tab))
+              if (r < body->h[V][0] || c < body->h[H][0])
                 xmlTextWriterWriteAttribute (odt->content_wtr, _xml("text:style-name"), _xml("Table_20_Heading"));
               else
                 xmlTextWriterWriteAttribute (odt->content_wtr, _xml("text:style-name"), _xml("Table_20_Contents"));
 
-              if (cell.options & TAB_MARKUP)
-                {
-                  /* XXX */
-                  char *s = output_get_text_from_markup (cell.text);
-                  write_xml_with_line_breaks (odt, s);
-                  free (s);
-                }
-              else
-                write_xml_with_line_breaks (odt, cell.text);
-
-              for (int i = 0; i < cell.n_footnotes; i++)
-                write_footnote (odt, cell.footnotes[i]);
+              write_table_item_cell (odt, pt, &cell);
 
               xmlTextWriterEndElement (odt->content_wtr); /* text:p */
               xmlTextWriterEndElement (odt->content_wtr); /* table:table-cell */
@@ -548,14 +551,30 @@ write_table (struct odt_driver *odt, const struct table_item *item)
 
       xmlTextWriterEndElement (odt->content_wtr); /* row */
 
-      if (table_ht (tab) > 0 && r == table_ht (tab) - 1)
+      int ht = body->h[V][0];
+      if (ht > 0 && r == ht - 1)
 	xmlTextWriterEndElement (odt->content_wtr); /* table-header-rows */
     }
 
   xmlTextWriterEndElement (odt->content_wtr); /* table */
 
   /* Write a caption for the table */
-  write_table_item_text (odt, table_item_get_caption (item));
+  write_table__ (odt, pt, caption);
+  write_table__ (odt, pt, footnotes);
+
+  table_unref (title);
+  table_unref (layers);
+  table_unref (body);
+  table_unref (caption);
+  table_unref (footnotes);
+}
+
+static void
+write_table (struct odt_driver *odt, const struct pivot_table *pt)
+{
+  size_t *layer_indexes;
+  PIVOT_OUTPUT_FOR_EACH_LAYER (layer_indexes, pt, true)
+    write_table_layer (odt, pt, layer_indexes);
 }
 
 static void
@@ -568,21 +587,43 @@ odt_output_text (struct odt_driver *odt, const char *text)
 
 /* Submit a table to the ODT driver */
 static void
-odt_submit (struct output_driver *driver,
-            const struct output_item *output_item)
+odt_submit (struct output_driver *driver, const struct output_item *item)
 {
   struct odt_driver *odt = odt_driver_cast (driver);
 
-  if (is_table_item (output_item))
-    write_table (odt, to_table_item (output_item));
-  else if (is_text_item (output_item))
-    odt_output_text (odt, text_item_get_text (to_text_item (output_item)));
-  else if (is_message_item (output_item))
+  switch (item->type)
     {
-      const struct message_item *message_item = to_message_item (output_item);
-      char *s = msg_to_string (message_item_get_msg (message_item));
-      odt_output_text (odt, s);
-      free (s);
+    case OUTPUT_ITEM_CHART:
+      break;
+
+    case OUTPUT_ITEM_GROUP:
+      break;
+
+    case OUTPUT_ITEM_IMAGE:
+      break;
+
+    case OUTPUT_ITEM_MESSAGE:
+      {
+        char *s = msg_to_string (item->message);
+        odt_output_text (odt, s);
+        free (s);
+      }
+      break;
+
+    case OUTPUT_ITEM_PAGE_BREAK:
+      break;
+
+    case OUTPUT_ITEM_TABLE:
+      write_table (odt, item->table);
+      break;
+
+    case OUTPUT_ITEM_TEXT:
+      {
+        char *text = text_item_get_plain_text (item);
+        odt_output_text (odt, text);
+        free (text);
+      }
+      break;
     }
 }
 
@@ -591,8 +632,7 @@ struct output_driver_factory odt_driver_factory =
 
 static const struct output_driver_class odt_driver_class =
 {
-  "odf",
-  odt_destroy,
-  odt_submit,
-  NULL,
+  .name = "odf",
+  .destroy = odt_destroy,
+  .submit = odt_submit,
 };
