@@ -19,16 +19,16 @@
 
 /* Tables.
 
-.  A table is a rectangular grid of cells.  Cells can be joined to form larger
+   A table is a rectangular grid of cells.  Cells can be joined to form larger
    cells.  Rows and columns can be separated by rules of various types.  Rows
    at the top and bottom of a table and columns at the left and right edges of
    a table can be designated as headers, which means that if the table must be
    broken across more than one page, those rows or columns are repeated on each
    page.
 
-   A table is not itself an output_item, and thus a table cannot by itself be
-   used for output, but they can be embedded inside struct table_item (see
-   table-item.h) for that purpose. */
+   Some drivers use tables as an implementation detail of rendering pivot
+   tables.
+*/
 
 #include <stdbool.h>
 #include <stdint.h>
@@ -37,6 +37,8 @@
 
 struct casereader;
 struct fmt_spec;
+struct pivot_footnote;
+struct pivot_value;
 struct pool;
 struct table_item;
 struct variable;
@@ -144,7 +146,7 @@ struct font_style
     bool bold, italic, underline, markup;
     struct cell_color fg[2], bg[2];
     char *typeface;
-    int size;
+    int size;                   /* In 1/72" units. */
   };
 
 #define FONT_STYLE_INITIALIZER { FONT_STYLE_INITIALIZER__ }
@@ -156,31 +158,30 @@ void font_style_copy (struct pool *,
                       struct font_style *, const struct font_style *);
 void font_style_uninit (struct font_style *);
 void font_style_dump (const struct font_style *);
+bool font_style_equal (const struct font_style *, const struct font_style *);
 
-struct area_style
+struct table_area_style
   {
     struct cell_style cell_style;
     struct font_style font_style;
   };
 
-#define AREA_STYLE_INITIALIZER { AREA_STYLE_INITIALIZER__ }
-#define AREA_STYLE_INITIALIZER__                \
+#define TABLE_AREA_STYLE_INITIALIZER { TABLE_AREA_STYLE_INITIALIZER__ }
+#define TABLE_AREA_STYLE_INITIALIZER__          \
        .cell_style = CELL_STYLE_INITIALIZER,    \
        .font_style = FONT_STYLE_INITIALIZER
 
-struct area_style *area_style_clone (struct pool *, const struct area_style *);
-void area_style_copy (struct pool *,
-                      struct area_style *, const struct area_style *);
-void area_style_uninit (struct area_style *);
-void area_style_free (struct area_style *);
+struct table_area_style *table_area_style_clone (
+  struct pool *, const struct table_area_style *);
+void table_area_style_copy (struct pool *, struct table_area_style *,
+                            const struct table_area_style *);
+void table_area_style_uninit (struct table_area_style *);
+void table_area_style_free (struct table_area_style *);
 
 /* Properties of a table cell. */
 enum
   {
     TAB_NONE = 0,
-    TAB_FIX        = 1 << 1,    /* Use fixed font. */
-    TAB_MARKUP     = 1 << 2,    /* Text contains Pango markup. */
-    TAB_NUMERIC    = 1 << 3,    /* Cell contents are numeric. */
     TAB_ROTATE     = 1 << 4,    /* Rotate cell contents 90 degrees. */
 
     TAB_STYLE_SHIFT = 5,
@@ -221,13 +222,13 @@ struct table
 
     /* Table contents.
 
-       Each array element in cc[] is ordinarily a "char *" pointer to a string.
+       Each array element in cc[] is ordinarily a "struct pivot_value *".
        If TAB_JOIN (defined in table.c) is set in ct[] for the element,
        however, it is a joined cell and the corresponding element of cc[]
        points to a struct table_cell. */
     void **cc;                  /* Cell contents; void *[nr][nc]. */
     unsigned short *ct;		/* Cell types; unsigned short[nr][nc]. */
-    struct area_style *styles[8];
+    struct table_area_style *styles[8];
 
     /* Rules. */
     unsigned char *rh;		/* Horiz rules; unsigned char[nr+1][nc]. */
@@ -240,29 +241,6 @@ struct table *table_ref (const struct table *);
 void table_unref (struct table *);
 bool table_is_shared (const struct table *);
 
-/* Returns the number of columns or rows, respectively, in T. */
-static inline int table_nc (const struct table *t)
-        { return t->n[TABLE_HORZ]; }
-static inline int table_nr (const struct table *t)
-        { return t->n[TABLE_VERT]; }
-
-/* Returns the number of left, right, top, or bottom headers, respectively, in
-   T.  */
-static inline int table_hl (const struct table *t)
-        { return t->h[TABLE_HORZ][0]; }
-static inline int table_hr (const struct table *t)
-        { return t->h[TABLE_HORZ][1]; }
-static inline int table_ht (const struct table *t)
-        { return t->h[TABLE_VERT][0]; }
-static inline int table_hb (const struct table *t)
-        { return t->h[TABLE_VERT][1]; }
-
-/* Simple kinds of output. */
-struct table *table_from_string (const char *);
-void table_output_text (int options, const char *string);
-void table_output_text_format (int options, const char *, ...)
-  PRINTF_FORMAT (2, 3);
-
 /* Rule masks. */
 #define TAB_RULE_TYPE_MASK   7
 #define TAB_RULE_TYPE_SHIFT  0
@@ -279,32 +257,10 @@ void table_box (struct table *, int f_h, int f_v, int i_h, int i_v,
                 int x1, int y1, int x2, int y2);
 
 /* Cells. */
-void table_text (struct table *, int c, int r, unsigned opt, const char *);
-void table_text_format (struct table *, int c, int r, unsigned opt,
-                        const char *, ...)
-  PRINTF_FORMAT (5, 6);
-void table_joint_text (struct table *, int x1, int y1, int x2, int y2,
-                       unsigned opt, const char *);
-
-void table_add_subscripts (struct table *, int x, int y,
-                           char **subscripts, size_t n_subscripts);
-void table_add_superscript (struct table *, int x, int y,
-                            const char *superscript);
-
-/* Footnotes.
-
-   Use table_create_footnote() to create the footnotes themselves, then use
-   table_add_footnote() to create a reference from a table cell to a footnote.
-   There are two steps because a footnote may have multiple references. */
-struct footnote *table_create_footnote (struct table *, size_t idx,
-                                        const char *content,
-                                        const char *marker,
-                                        struct area_style *);
-void table_add_footnote (struct table *, int x, int y,
-                         const struct footnote *);
-
-void table_add_style (struct table *, int x, int y,
-                      const struct area_style *);
+void table_put (struct table *, int x1, int y1, int x2, int y2,
+                unsigned opt, const struct pivot_value *);
+void table_put_owned (struct table *, int x1, int y1, int x2, int y2,
+                      unsigned opt, struct pivot_value *);
 
 bool table_cell_is_empty (const struct table *, int c, int r);
 
