@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2017, 2019  John Darrington
+   Copyright (C) 2017, 2019, 2020  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "ui/gui/executor.h"
 #include "psppire-data-window.h"
 #include "ssw-axis-model.h"
+#include "helper.h"
 
 static void
 do_sort (PsppireDataSheet *sheet, GtkSortType order)
@@ -372,10 +373,28 @@ psppire_data_sheet_dispose (GObject *obj)
   G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
+
+static void
+psppire_data_sheet_realize (GtkWidget *widget)
+{
+  g_object_set (widget,
+                "forward-conversion", psppire_data_store_value_to_string,
+                "reverse-conversion", psppire_data_store_string_to_value,
+                "editable", TRUE,
+                "horizontal-draggable", TRUE,
+                NULL);
+
+  /* Chain up to the parent class */
+  GTK_WIDGET_CLASS (parent_class)->realize (widget);
+}
+
 static void
 psppire_data_sheet_class_init (PsppireDataSheetClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
+
+  widget_class->realize = psppire_data_sheet_realize;
   object_class->dispose = psppire_data_sheet_dispose;
   object_class->finalize = psppire_data_sheet_finalize;
 
@@ -385,15 +404,7 @@ psppire_data_sheet_class_init (PsppireDataSheetClass *class)
 GtkWidget*
 psppire_data_sheet_new (void)
 {
-  GObject *obj =
-    g_object_new (PSPPIRE_TYPE_DATA_SHEET,
-		  "forward-conversion", psppire_data_store_value_to_string,
-		  "reverse-conversion", psppire_data_store_string_to_value,
-		  "editable", TRUE,
-		  "horizontal-draggable", TRUE,
-		  NULL);
-
-  return GTK_WIDGET (obj);
+  return g_object_new (PSPPIRE_TYPE_DATA_SHEET, NULL);
 }
 
 
@@ -424,7 +435,6 @@ button_post_create (GtkWidget *button, guint i, gpointer user_data)
   g_signal_connect_after (button, "draw", G_CALLBACK (indicate_filtered_case), data_store);
 }
 
-
 static gboolean
 resize_display_width (PsppireDict *dict, gint pos, gint size, gpointer user_data)
 {
@@ -432,19 +442,9 @@ resize_display_width (PsppireDict *dict, gint pos, gint size, gpointer user_data
     return FALSE;
 
   PsppireDataSheet *sheet = PSPPIRE_DATA_SHEET (user_data);
-  PangoContext *context = gtk_widget_create_pango_context (GTK_WIDGET (sheet));
-  PangoLayout *layout = pango_layout_new (context);
-  PangoRectangle rect;
+  gdouble wm = width_of_m (GTK_WIDGET (sheet));
 
-  pango_layout_set_text (layout, "M", 1);
-  pango_layout_get_extents (layout, NULL, &rect);
-
-  gdouble width_of_M = rect.width / (gdouble) PANGO_SCALE;
-
-  g_object_unref (G_OBJECT (layout));
-  g_object_unref (G_OBJECT (context));
-
-  gint Ms = round ((size / width_of_M) - 0.25);
+  gint Ms = round ((size / wm) - 0.25);
   struct variable *var = psppire_dict_get_variable (dict, pos);
   g_return_val_if_fail (var, TRUE);
   var_set_display_width (var, Ms);
@@ -457,6 +457,8 @@ set_dictionary (PsppireDataSheet *sheet)
   GtkTreeModel *data_model = NULL;
   g_object_get (sheet, "data-model", &data_model, NULL);
 
+  g_return_if_fail (data_model);
+
   PsppireDataStore *store = PSPPIRE_DATA_STORE (data_model);
   g_object_set (sheet, "hmodel", store->dict, NULL);
 
