@@ -98,7 +98,7 @@ __tree_model_get_n_columns (GtkTreeModel *tree_model)
 {
   PsppireDataStore *store  = PSPPIRE_DATA_STORE (tree_model);
 
-  return psppire_dict_get_var_cnt (store->dict);
+  return psppire_dict_get_n_vars (store->dict);
 }
 
 
@@ -134,11 +134,11 @@ psppire_data_store_string_to_value (GtkTreeModel *model, gint col, gint row,
 {
   PsppireDataStore *store = PSPPIRE_DATA_STORE (model);
 
-  while (col >= psppire_dict_get_var_cnt (store->dict))
+  while (col >= psppire_dict_get_n_vars (store->dict))
     {
       const struct variable *var =
 	psppire_dict_insert_variable (store->dict,
-				      psppire_dict_get_var_cnt (store->dict),
+				      psppire_dict_get_n_vars (store->dict),
 				      NULL);
       g_return_val_if_fail (var, FALSE);
     }
@@ -164,7 +164,8 @@ psppire_data_store_string_to_value (GtkTreeModel *model, gint col, gint row,
   if (vp == NULL)
     {
       xx = data_in (ss_cstr (in), psppire_dict_encoding (store->dict),
-		    fmt->type, &val, width, "UTF-8");
+		    fmt->type, settings_get_fmt_settings (),
+                    &val, width, "UTF-8");
     }
 
   GVariant *vrnt = value_variant_new (&val, width);
@@ -180,7 +181,7 @@ static char *
 unlabeled_value (PsppireDataStore *store, const struct variable *variable, const union value *val)
 {
   if (var_is_numeric (variable) &&
-      var_is_value_missing (variable, val, MV_SYSTEM))
+      var_is_value_missing (variable, val) == MV_SYSTEM)
     return g_strdup ("");
 
   const struct fmt_spec *fmt = var_get_print_format (variable);
@@ -331,7 +332,7 @@ psppire_data_store_get_case_count (const PsppireDataStore *store)
 size_t
 psppire_data_store_get_value_count (const PsppireDataStore *store)
 {
-  return psppire_dict_get_value_cnt (store->dict);
+  return psppire_dict_get_n_values (store->dict);
 }
 
 const struct caseproto *
@@ -388,11 +389,13 @@ resize_datum (const union value *old, union value *new, const void *aux_)
   int new_width = var_get_width (aux->new_variable);
   const char *enc = dict_get_encoding (aux->dict);
   const struct fmt_spec *newfmt = var_get_print_format (aux->new_variable);
-  char *s = data_out (old, enc, var_get_print_format (aux->old_variable));
+  char *s = data_out (old, enc, var_get_print_format (aux->old_variable),
+                      settings_get_fmt_settings ());
   enum fmt_type type = (fmt_usable_for_input (newfmt->type)
                         ? newfmt->type
                         : FMT_DOLLAR);
-  free (data_in (ss_cstr (s), enc, type, new, new_width, enc));
+  free (data_in (ss_cstr (s), enc, type, settings_get_fmt_settings (),
+                 new, new_width, enc));
   free (s);
 }
 
@@ -704,11 +707,10 @@ psppire_data_store_clear (PsppireDataStore *ds)
 struct casereader *
 psppire_data_store_get_reader (PsppireDataStore *ds)
 {
-  int i;
   struct casereader *reader ;
 
   if (ds->dict)
-    for (i = 0 ; i < n_dict_signals; ++i)
+    for (int i = 0 ; i < n_dict_signals; ++i)
       {
 	g_signal_handler_block (ds->dict,
 				ds->dict_handler_id[i]);
@@ -838,8 +840,8 @@ psppire_data_store_data_in (PsppireDataStore *ds, casenumber casenum, gint idx,
                         FALSE);
   value_init (&value, width);
   ok = (datasheet_get_value (ds->datasheet, casenum, idx, &value)
-        && data_in_msg (input, UTF8, fmt->type, &value, width,
-                        dict_get_encoding (dict->dict))
+        && data_in_msg (input, UTF8, fmt->type, settings_get_fmt_settings (),
+                        &value, width, dict_get_encoding (dict->dict))
         && datasheet_put_value (ds->datasheet, casenum, idx, &value));
   value_destroy (&value, width);
 
