@@ -394,7 +394,7 @@ psppire_window_init (PsppireWindow *window)
 
   g_signal_connect_swapped (window, "delete-event", G_CALLBACK (on_delete), window);
 
-  g_object_set (window, "icon-name", "pspp", NULL);
+  g_object_set (window, "icon-name", "org.gnu.pspp", NULL);
 }
 
 /*
@@ -429,7 +429,7 @@ psppire_window_query_save (PsppireWindow *se)
 			    description);
   g_free (description);
 
-  g_object_set (dialog, "icon-name", "pspp", NULL);
+  g_object_set (dialog, "icon-name", "org.gnu.pspp", NULL);
 
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 					    _("If you don't save, changes from the last %ld seconds will be permanently lost."),
